@@ -74,7 +74,7 @@ NET_API_STATUS NetJoinDomain(const char * server /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -123,7 +123,7 @@ NET_API_STATUS NetUnjoinDomain(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -170,7 +170,7 @@ NET_API_STATUS NetGetJoinInformation(const char * server_name /* [in] [unique] *
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -223,7 +223,7 @@ NET_API_STATUS NetGetJoinableOUs(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -274,7 +274,7 @@ NET_API_STATUS NetRenameMachineInDomain(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -321,7 +321,7 @@ NET_API_STATUS NetServerGetInfo(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -370,7 +370,7 @@ NET_API_STATUS NetServerSetInfo(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -417,7 +417,7 @@ NET_API_STATUS NetGetDCName(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -464,7 +464,7 @@ NET_API_STATUS NetGetAnyDCName(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -517,7 +517,7 @@ NET_API_STATUS DsGetDcName(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -566,7 +566,7 @@ NET_API_STATUS NetUserAdd(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -611,7 +611,7 @@ NET_API_STATUS NetUserDel(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -669,7 +669,7 @@ NET_API_STATUS NetUserEnum(const char * server_name /* [in] [unique] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -718,7 +718,7 @@ NET_API_STATUS NetUserChangePassword(const char * domain_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -767,7 +767,7 @@ NET_API_STATUS NetUserGetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -818,7 +818,7 @@ NET_API_STATUS NetUserSetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -873,7 +873,7 @@ NET_API_STATUS NetUserGetGroups(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -924,7 +924,7 @@ NET_API_STATUS NetUserSetGroups(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -981,7 +981,7 @@ NET_API_STATUS NetUserGetLocalGroups(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1028,7 +1028,7 @@ NET_API_STATUS NetUserModalsGet(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1077,7 +1077,7 @@ NET_API_STATUS NetUserModalsSet(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1132,7 +1132,7 @@ NET_API_STATUS NetQueryDisplayInformation(const char * server_name /* [in] [uniq
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1181,7 +1181,7 @@ NET_API_STATUS NetGroupAdd(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1226,7 +1226,7 @@ NET_API_STATUS NetGroupDel(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1282,7 +1282,7 @@ NET_API_STATUS NetGroupEnum(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1333,7 +1333,7 @@ NET_API_STATUS NetGroupSetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1382,7 +1382,7 @@ NET_API_STATUS NetGroupGetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1429,7 +1429,7 @@ NET_API_STATUS NetGroupAddUser(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1476,7 +1476,7 @@ NET_API_STATUS NetGroupDelUser(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1534,7 +1534,7 @@ NET_API_STATUS NetGroupGetUsers(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1585,7 +1585,7 @@ NET_API_STATUS NetGroupSetUsers(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1634,7 +1634,7 @@ NET_API_STATUS NetLocalGroupAdd(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1679,7 +1679,7 @@ NET_API_STATUS NetLocalGroupDel(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1728,7 +1728,7 @@ NET_API_STATUS NetLocalGroupGetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1779,7 +1779,7 @@ NET_API_STATUS NetLocalGroupSetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1835,7 +1835,7 @@ NET_API_STATUS NetLocalGroupEnum(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1886,7 +1886,7 @@ NET_API_STATUS NetLocalGroupAddMembers(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1937,7 +1937,7 @@ NET_API_STATUS NetLocalGroupDelMembers(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -1995,7 +1995,7 @@ NET_API_STATUS NetLocalGroupGetMembers(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2046,7 +2046,7 @@ NET_API_STATUS NetLocalGroupSetMembers(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2091,7 +2091,7 @@ NET_API_STATUS NetRemoteTOD(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2140,7 +2140,7 @@ NET_API_STATUS NetShareAdd(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2187,7 +2187,7 @@ NET_API_STATUS NetShareDel(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2243,7 +2243,7 @@ NET_API_STATUS NetShareEnum(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2292,7 +2292,7 @@ NET_API_STATUS NetShareGetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2343,7 +2343,7 @@ NET_API_STATUS NetShareSetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2388,7 +2388,7 @@ NET_API_STATUS NetFileClose(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2437,7 +2437,7 @@ NET_API_STATUS NetFileGetInfo(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2497,7 +2497,7 @@ NET_API_STATUS NetFileEnum(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2548,7 +2548,7 @@ NET_API_STATUS NetShutdownInit(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2591,7 +2591,7 @@ NET_API_STATUS NetShutdownAbort(const char * server_name /* [in] */)
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2640,7 +2640,7 @@ NET_API_STATUS I_NetLogonControl(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
 /****************************************************************
@@ -2691,6 +2691,6 @@ NET_API_STATUS I_NetLogonControl2(const char * server_name /* [in] */,
 	}
 
 	TALLOC_FREE(frame);
-	return r.out.result;
+	return (NET_API_STATUS)r.out.result;
 }
 
