@@ -29,6 +29,7 @@
 #include "../libcli/security/security.h"
 #include "passdb/machine_sid.h"
 #include "smbd/smbd.h"
+#include "smbd/globals.h"
 #include "auth.h"
 #include "messages.h"
 #include "rpc_server/spoolss/srv_spoolss_nt.h"
@@ -73,6 +74,63 @@ static const struct print_architecture_table_node archi_table[]= {
 	{NULL,                   "",		-1 }
 };
 
+static bool print_driver_directories_init(void)
+{
+	int service, i;
+	char *driver_path;
+	bool ok;
+	TALLOC_CTX *mem_ctx = talloc_stackframe();
+
+	service = lp_servicenumber("print$");
+	if (service < 0) {
+		/* We don't have a print$ share */
+		DEBUG(5, ("No print$ share has been configured.\n"));
+		talloc_free(mem_ctx);
+		return true;
+	}
+
+	driver_path = lp_pathname(mem_ctx, service);
+	if (driver_path == NULL) {
+		talloc_free(mem_ctx);
+		return false;
+	}
+
+	ok = directory_create_or_exist(driver_path, sec_initial_uid(), 0755);
+	if (!ok) {
+		DEBUG(1, ("Failed to create printer driver directory %s\n",
+			  driver_path));
+		talloc_free(mem_ctx);
+		return false;
+	}
+
+	for (i = 0; archi_table[i].long_archi != NULL; i++) {
+		const char *arch_path;
+
+		arch_path = talloc_asprintf(mem_ctx,
+					    "%s/%s",
+					    driver_path,
+					    archi_table[i].short_archi);
+		if (arch_path == NULL) {
+			talloc_free(mem_ctx);
+			return false;
+		}
+
+		ok = directory_create_or_exist(arch_path,
+					       sec_initial_uid(),
+					       0755);
+		if (!ok) {
+			DEBUG(1, ("Failed to create printer driver "
+				  "architecture directory %s\n",
+				  arch_path));
+			talloc_free(mem_ctx);
+			return false;
+		}
+	}
+
+	talloc_free(mem_ctx);
+	return true;
+}
+
 /****************************************************************************
  Open the NT printing tdbs. Done once before fork().
 ****************************************************************************/
@@ -81,6 +139,10 @@ bool nt_printing_init(struct messaging_context *msg_ctx)
 {
 	WERROR win_rc;
 
+	if (!print_driver_directories_init()) {
+		return false;
+	}
+
 	if (!nt_printing_tdb_upgrade()) {
 		return false;
 	}
@@ -148,7 +210,7 @@ const char *get_short_archi(const char *long_archi)
         do {
                 i++;
         } while ( (archi_table[i].long_archi!=NULL ) &&
-                  StrCaseCmp(long_archi, archi_table[i].long_archi) );
+                  strcasecmp_m(long_archi, archi_table[i].long_archi) );
 
         if (archi_table[i].long_archi==NULL) {
                 DEBUGADD(10,("Unknown architecture [%s] !\n", long_archi));
@@ -198,7 +260,7 @@ static int get_file_version(files_struct *fsp, char *fname,uint32 *major, uint32
 	}
 
 	/* Skip OEM header (if any) and the DOS stub to start of Windows header */
-	if (SMB_VFS_LSEEK(fsp, SVAL(buf,DOS_HEADER_LFANEW_OFFSET), SEEK_SET) == (SMB_OFF_T)-1) {
+	if (SMB_VFS_LSEEK(fsp, SVAL(buf,DOS_HEADER_LFANEW_OFFSET), SEEK_SET) == (off_t)-1) {
 		DEBUG(3,("get_file_version: File [%s] too short, errno = %d\n",
 				fname, errno));
 		/* Assume this isn't an error... the file just looks sort of like a PE/NE file */
@@ -221,7 +283,7 @@ static int get_file_version(files_struct *fsp, char *fname,uint32 *major, uint32
 		/* Just skip over optional header to get to section table */
 		if (SMB_VFS_LSEEK(fsp,
 				SVAL(buf,PE_HEADER_OPTIONAL_HEADER_SIZE)-(NE_HEADER_SIZE-PE_HEADER_SIZE),
-				SEEK_CUR) == (SMB_OFF_T)-1) {
+				SEEK_CUR) == (off_t)-1) {
 			DEBUG(3,("get_file_version: File [%s] Windows optional header too short, errno = %d\n",
 				fname, errno));
 			goto error_exit;
@@ -265,7 +327,7 @@ static int get_file_version(files_struct *fsp, char *fname,uint32 *major, uint32
 				}
 
 				/* Seek to the start of the .rsrc section info */
-				if (SMB_VFS_LSEEK(fsp, section_pos, SEEK_SET) == (SMB_OFF_T)-1) {
+				if (SMB_VFS_LSEEK(fsp, section_pos, SEEK_SET) == (off_t)-1) {
 					DEBUG(3,("get_file_version: PE file [%s] too short for section info, errno = %d\n",
 							fname, errno));
 					goto error_exit;
@@ -574,7 +636,7 @@ static int file_version_is_newer(connection_struct *conn, fstring new_file, fstr
 /****************************************************************************
 Determine the correct cVersion associated with an architecture and driver
 ****************************************************************************/
-static uint32 get_correct_cversion(struct auth_serversupplied_info *session_info,
+static uint32 get_correct_cversion(struct auth_session_info *session_info,
 				   const char *architecture,
 				   const char *driverpath_in,
 				   WERROR *perr)
@@ -615,9 +677,13 @@ static uint32 get_correct_cversion(struct auth_serversupplied_info *session_info
 		return -1;
 	}
 
-	nt_status = create_conn_struct(talloc_tos(), &conn, printdollar_snum,
-				       lp_pathname(printdollar_snum),
-				       session_info, &oldcwd);
+	nt_status = create_conn_struct_cwd(talloc_tos(),
+					   server_event_context(),
+					   server_messaging_context(),
+					   &conn,
+					   printdollar_snum,
+					   lp_pathname(talloc_tos(), printdollar_snum),
+					   session_info, &oldcwd);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUG(0,("get_correct_cversion: create_conn_struct "
 			 "returned %s\n", nt_errstr(nt_status)));
@@ -744,7 +810,7 @@ static uint32 get_correct_cversion(struct auth_serversupplied_info *session_info
 		SMB_VFS_DISCONNECT(conn);
 		conn_free(conn);
 	}
-	if (!NT_STATUS_IS_OK(*perr)) {
+	if (!W_ERROR_IS_OK(*perr)) {
 		cversion = -1;
 	}
 
@@ -762,7 +828,7 @@ static uint32 get_correct_cversion(struct auth_serversupplied_info *session_info
 } while (0);
 
 static WERROR clean_up_driver_struct_level(TALLOC_CTX *mem_ctx,
-					   struct auth_serversupplied_info *session_info,
+					   struct auth_session_info *session_info,
 					   const char *architecture,
 					   const char **driver_path,
 					   const char **data_file,
@@ -835,7 +901,7 @@ static WERROR clean_up_driver_struct_level(TALLOC_CTX *mem_ctx,
 ****************************************************************************/
 
 WERROR clean_up_driver_struct(TALLOC_CTX *mem_ctx,
-			      struct auth_serversupplied_info *session_info,
+			      struct auth_session_info *session_info,
 			      struct spoolss_AddDriverInfoCtr *r)
 {
 	switch (r->level) {
@@ -920,7 +986,7 @@ static WERROR move_driver_file_to_download_area(TALLOC_CTX *mem_ctx,
 		}
 
 		/* Setup a synthetic smb_filename struct */
-		smb_fname_new = TALLOC_ZERO_P(mem_ctx, struct smb_filename);
+		smb_fname_new = talloc_zero(mem_ctx, struct smb_filename);
 		if (!smb_fname_new) {
 			ret = WERR_NOMEM;
 			goto out;
@@ -954,7 +1020,7 @@ static WERROR move_driver_file_to_download_area(TALLOC_CTX *mem_ctx,
 	return ret;
 }
 
-WERROR move_driver_to_download_area(struct auth_serversupplied_info *session_info,
+WERROR move_driver_to_download_area(struct auth_session_info *session_info,
 				    struct spoolss_AddDriverInfoCtr *r)
 {
 	struct spoolss_AddDriverInfo3 *driver;
@@ -998,9 +1064,13 @@ WERROR move_driver_to_download_area(struct auth_serversupplied_info *session_inf
 		return WERR_NO_SUCH_SHARE;
 	}
 
-	nt_status = create_conn_struct(talloc_tos(), &conn, printdollar_snum,
-				       lp_pathname(printdollar_snum),
-				       session_info, &oldcwd);
+	nt_status = create_conn_struct_cwd(talloc_tos(),
+					   server_event_context(),
+					   server_messaging_context(),
+					   &conn,
+					   printdollar_snum,
+					   lp_pathname(talloc_tos(), printdollar_snum),
+					   session_info, &oldcwd);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUG(0,("move_driver_to_download_area: create_conn_struct "
 			 "returned %s\n", nt_errstr(nt_status)));
@@ -1174,16 +1244,14 @@ WERROR move_driver_to_download_area(struct auth_serversupplied_info *session_inf
 ****************************************************************************/
 
 bool printer_driver_in_use(TALLOC_CTX *mem_ctx,
-			   const struct auth_serversupplied_info *session_info,
-			   struct messaging_context *msg_ctx,
-                           const struct spoolss_DriverInfo8 *r)
+			   struct dcerpc_binding_handle *b,
+			   const struct spoolss_DriverInfo8 *r)
 {
 	int snum;
 	int n_services = lp_numservices();
 	bool in_use = False;
 	struct spoolss_PrinterInfo2 *pinfo2 = NULL;
 	WERROR result;
-	struct dcerpc_binding_handle *b = NULL;
 
 	if (!r) {
 		return false;
@@ -1198,18 +1266,8 @@ bool printer_driver_in_use(TALLOC_CTX *mem_ctx,
 			continue;
 		}
 
-		if (b == NULL) {
-			result = winreg_printer_binding_handle(mem_ctx,
-							       session_info,
-							       msg_ctx,
-							       &b);
-			if (!W_ERROR_IS_OK(result)) {
-				return false;
-			}
-		}
-
 		result = winreg_get_printer(mem_ctx, b,
-					    lp_servicename(snum),
+					    lp_servicename(talloc_tos(), snum),
 					    &pinfo2);
 		if (!W_ERROR_IS_OK(result)) {
 			continue; /* skip */
@@ -1225,7 +1283,7 @@ bool printer_driver_in_use(TALLOC_CTX *mem_ctx,
 	DEBUG(10,("printer_driver_in_use: Completed search through ntprinters.tdb...\n"));
 
 	if ( in_use ) {
-		struct spoolss_DriverInfo8 *driver;
+		struct spoolss_DriverInfo8 *driver = NULL;
 		WERROR werr;
 
 		DEBUG(5,("printer_driver_in_use: driver \"%s\" is currently in use\n", r->driver_name));
@@ -1413,8 +1471,7 @@ static bool trim_overlap_drv_files(TALLOC_CTX *mem_ctx,
 ****************************************************************************/
 
 bool printer_driver_files_in_use(TALLOC_CTX *mem_ctx,
-				 const struct auth_serversupplied_info *session_info,
-				 struct messaging_context *msg_ctx,
+				 struct dcerpc_binding_handle *b,
 				 struct spoolss_DriverInfo8 *info)
 {
 	int 				i;
@@ -1424,7 +1481,6 @@ bool printer_driver_files_in_use(TALLOC_CTX *mem_ctx,
 	uint32_t num_drivers;
 	const char **drivers;
 	WERROR result;
-	struct dcerpc_binding_handle *b;
 
 	if ( !info )
 		return False;
@@ -1437,14 +1493,6 @@ bool printer_driver_files_in_use(TALLOC_CTX *mem_ctx,
 
 	/* get the list of drivers */
 
-	result = winreg_printer_binding_handle(mem_ctx,
-					       session_info,
-					       msg_ctx,
-					       &b);
-	if (!W_ERROR_IS_OK(result)) {
-		return false;
-	}
-
 	result = winreg_get_driver_list(mem_ctx, b,
 					info->architecture, version,
 					&num_drivers, &drivers);
@@ -1508,9 +1556,8 @@ static NTSTATUS driver_unlink_internals(connection_struct *conn,
 		goto err_out;
 	}
 
-	status = create_synthetic_smb_fname(tmp_ctx, print_dlr_path,
-					    NULL, NULL, &smb_fname);
-	if (!NT_STATUS_IS_OK(status)) {
+	smb_fname = synthetic_smb_fname(tmp_ctx, print_dlr_path, NULL, NULL);
+	if (smb_fname == NULL) {
 		goto err_out;
 	}
 
@@ -1526,7 +1573,7 @@ err_out:
   this.
 ****************************************************************************/
 
-bool delete_driver_files(const struct auth_serversupplied_info *session_info,
+bool delete_driver_files(const struct auth_session_info *session_info,
 			 const struct spoolss_DriverInfo8 *r)
 {
 	const char *short_arch;
@@ -1552,9 +1599,13 @@ bool delete_driver_files(const struct auth_serversupplied_info *session_info,
 		return false;
 	}
 
-	nt_status = create_conn_struct(talloc_tos(), &conn, printdollar_snum,
-				       lp_pathname(printdollar_snum),
-				       session_info, &oldcwd);
+	nt_status = create_conn_struct_cwd(talloc_tos(),
+					   server_event_context(),
+					   server_messaging_context(),
+					   &conn,
+					   printdollar_snum,
+					   lp_pathname(talloc_tos(), printdollar_snum),
+					   session_info, &oldcwd);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUG(0,("delete_driver_files: create_conn_struct "
 			 "returned %s\n", nt_errstr(nt_status)));
@@ -1636,7 +1687,7 @@ bool delete_driver_files(const struct auth_serversupplied_info *session_info,
 	2: file doesn't exist
 	3: can't allocate memory
 	4: can't free memory
-	5: non existant struct
+	5: non existent struct
 */
 
 /*
@@ -1714,7 +1765,7 @@ void map_job_permissions(struct security_descriptor *sd)
     3)  "printer admins" (may result in numerous calls to winbind)
 
  ****************************************************************************/
-bool print_access_check(const struct auth_serversupplied_info *session_info,
+bool print_access_check(const struct auth_session_info *session_info,
 			struct messaging_context *msg_ctx, int snum,
 			int access_type)
 {
@@ -1730,14 +1781,14 @@ bool print_access_check(const struct auth_serversupplied_info *session_info,
 
 	/* Always allow root or SE_PRINT_OPERATROR to do anything */
 
-	if (session_info->utok.uid == sec_initial_uid()
+	if (session_info->unix_token->uid == sec_initial_uid()
 	    || security_token_has_privilege(session_info->security_token, SEC_PRIV_PRINT_OPERATOR)) {
 		return True;
 	}
 
 	/* Get printer name */
 
-	pname = lp_printername(snum);
+	pname = lp_printername(talloc_tos(), snum);
 
 	if (!pname || !*pname) {
 		errno = EACCES;
@@ -1792,17 +1843,6 @@ bool print_access_check(const struct auth_serversupplied_info *session_info,
 
 	DEBUG(4, ("access check was %s\n", NT_STATUS_IS_OK(status) ? "SUCCESS" : "FAILURE"));
 
-        /* see if we need to try the printer admin list */
-
-        if (!NT_STATUS_IS_OK(status) &&
-	    (token_contains_name_in_list(uidtoname(session_info->utok.uid),
-					 session_info->info3->base.domain.string,
-					 NULL, session_info->security_token,
-					 lp_printer_admin(snum)))) {
-		talloc_destroy(mem_ctx);
-		return True;
-        }
-
 	talloc_destroy(mem_ctx);
 
 	if (!NT_STATUS_IS_OK(status)) {
@@ -1816,7 +1856,7 @@ bool print_access_check(const struct auth_serversupplied_info *session_info,
  Check the time parameters allow a print operation.
 *****************************************************************************/
 
-bool print_time_access_check(const struct auth_serversupplied_info *session_info,
+bool print_time_access_check(const struct auth_session_info *session_info,
 			     struct messaging_context *msg_ctx,
 			     const char *servicename)
 {
@@ -1854,7 +1894,7 @@ bool print_time_access_check(const struct auth_serversupplied_info *session_info
 }
 
 void nt_printer_remove(TALLOC_CTX *mem_ctx,
-			const struct auth_serversupplied_info *session_info,
+			const struct auth_session_info *session_info,
 			struct messaging_context *msg_ctx,
 			const char *printer)
 {
@@ -1864,12 +1904,12 @@ void nt_printer_remove(TALLOC_CTX *mem_ctx,
 					   printer, "");
 	if (!W_ERROR_IS_OK(result)) {
 		DEBUG(0, ("nt_printer_remove: failed to remove printer %s: "
-			  "%s\n", printer, win_errstr(result)));
+		"%s\n", printer, win_errstr(result)));
 	}
 }
 
 void nt_printer_add(TALLOC_CTX *mem_ctx,
-		    const struct auth_serversupplied_info *session_info,
+		    const struct auth_session_info *session_info,
 		    struct messaging_context *msg_ctx,
 		    const char *printer)
 {
