@@ -18,7 +18,9 @@
 */
 
 #include "includes.h"
+#include "smbd/smbd.h"
 #include "smbd/globals.h"
+#include "messages.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_LOCKING
@@ -33,25 +35,33 @@ static void received_unlock_msg(struct messaging_context *msg,
 				struct server_id server_id,
 				DATA_BLOB *data);
 
-static void brl_timeout_fn(struct event_context *event_ctx,
-			   struct timed_event *te,
+void brl_timeout_fn(struct tevent_context *event_ctx,
+			   struct tevent_timer *te,
 			   struct timeval now,
 			   void *private_data)
 {
-	SMB_ASSERT(brl_timeout == te);
-	TALLOC_FREE(brl_timeout);
+	struct smbd_server_connection *sconn = talloc_get_type_abort(
+		private_data, struct smbd_server_connection);
+
+	if (sconn->using_smb2) {
+		SMB_ASSERT(sconn->smb2.locks.brl_timeout == te);
+		TALLOC_FREE(sconn->smb2.locks.brl_timeout);
+	} else {
+		SMB_ASSERT(sconn->smb1.locks.brl_timeout == te);
+		TALLOC_FREE(sconn->smb1.locks.brl_timeout);
+	}
 
 	change_to_root_user();	/* TODO: Possibly run all timed events as
 				 * root */
 
-	process_blocking_lock_queue();
+	process_blocking_lock_queue(sconn);
 }
 
 /****************************************************************************
  We need a version of timeval_min that treats zero timval as infinite.
 ****************************************************************************/
 
-static struct timeval timeval_brl_min(const struct timeval *tv1,
+struct timeval timeval_brl_min(const struct timeval *tv1,
 					const struct timeval *tv2)
 {
 	if (timeval_is_zero(tv1)) {
@@ -68,24 +78,24 @@ static struct timeval timeval_brl_min(const struct timeval *tv1,
  next processing.
 ****************************************************************************/
 
-static bool recalc_brl_timeout(void)
+static bool recalc_brl_timeout(struct smbd_server_connection *sconn)
 {
 	struct blocking_lock_record *blr;
 	struct timeval next_timeout;
 	int max_brl_timeout = lp_parm_int(-1, "brl", "recalctime", 5);
 
-	TALLOC_FREE(brl_timeout);
+	TALLOC_FREE(sconn->smb1.locks.brl_timeout);
 
-	next_timeout = timeval_zero();	
+	next_timeout = timeval_zero();
 
-	for (blr = blocking_lock_queue; blr; blr = blr->next) {
+	for (blr = sconn->smb1.locks.blocking_lock_queue; blr; blr = blr->next) {
 		if (timeval_is_zero(&blr->expire_time)) {
 			/*
-			 * If we're blocked on pid 0xFFFFFFFF this is
+			 * If we're blocked on pid 0xFFFFFFFFFFFFFFFFLL this is
 			 * a POSIX lock, so calculate a timeout of
 			 * 10 seconds into the future.
 			 */
-                        if (blr->blocking_pid == 0xFFFFFFFF) {
+                        if (blr->blocking_smblctx == 0xFFFFFFFFFFFFFFFFLL) {
 				struct timeval psx_to = timeval_current_ofs(10, 0);
 				next_timeout = timeval_brl_min(&next_timeout, &psx_to);
                         }
@@ -117,7 +127,7 @@ static bool recalc_brl_timeout(void)
 
 	if (max_brl_timeout > 0) {
 		struct timeval min_to = timeval_current_ofs(max_brl_timeout, 0);
-		next_timeout = timeval_min(&next_timeout, &min_to);             
+		next_timeout = timeval_min(&next_timeout, &min_to);
 	}
 
 	if (DEBUGLVL(10)) {
@@ -129,9 +139,10 @@ static bool recalc_brl_timeout(void)
 		    (int)from_now.tv_sec, (int)from_now.tv_usec));
 	}
 
-	if (!(brl_timeout = event_add_timed(smbd_event_context(), NULL,
-					    next_timeout,
-					    brl_timeout_fn, NULL))) {
+	sconn->smb1.locks.brl_timeout = tevent_add_timer(sconn->ev_ctx,
+							 NULL, next_timeout,
+							 brl_timeout_fn, sconn);
+	if (sconn->smb1.locks.brl_timeout == NULL) {
 		return False;
 	}
 
@@ -148,16 +159,31 @@ bool push_blocking_lock_request( struct byte_range_lock *br_lck,
 		files_struct *fsp,
 		int lock_timeout,
 		int lock_num,
-		uint32_t lock_pid,
+		uint64_t smblctx,
 		enum brl_type lock_type,
 		enum brl_flavour lock_flav,
 		uint64_t offset,
 		uint64_t count,
-		uint32_t blocking_pid)
+		uint64_t blocking_smblctx)
 {
+	struct smbd_server_connection *sconn = req->sconn;
 	struct blocking_lock_record *blr;
 	NTSTATUS status;
 
+	if (req->smb2req) {
+		return push_blocking_lock_request_smb2(br_lck,
+				req,
+				fsp,
+				lock_timeout,
+				lock_num,
+				smblctx,
+				lock_type,
+				lock_flav,
+				offset,
+				count,
+				blocking_smblctx);
+	}
+
 	if(req_is_in_chain(req)) {
 		DEBUG(0,("push_blocking_lock_request: cannot queue a chained request (currently).\n"));
 		return False;
@@ -182,12 +208,11 @@ bool push_blocking_lock_request( struct byte_range_lock *br_lck,
 		blr->expire_time.tv_sec = 0;
 		blr->expire_time.tv_usec = 0; /* Never expire. */
 	} else {
-		blr->expire_time = timeval_current_ofs(lock_timeout/1000,
-					(lock_timeout % 1000) * 1000);
+		blr->expire_time = timeval_current_ofs_msec(lock_timeout);
 	}
 	blr->lock_num = lock_num;
-	blr->lock_pid = lock_pid;
-	blr->blocking_pid = blocking_pid;
+	blr->smblctx = smblctx;
+	blr->blocking_smblctx = blocking_smblctx;
 	blr->lock_flav = lock_flav;
 	blr->lock_type = lock_type;
 	blr->offset = offset;
@@ -197,10 +222,10 @@ bool push_blocking_lock_request( struct byte_range_lock *br_lck,
 	blr->blr_private = NULL;
 
 	/* Add a pending lock record for this. */
-	status = brl_lock(smbd_messaging_context(),
+	status = brl_lock(req->sconn->msg_ctx,
 			br_lck,
-			lock_pid,
-			procid_self(),
+			smblctx,
+			messaging_server_id(req->sconn->msg_ctx),
 			offset,
 			count,
 			lock_type == READ_LOCK ? PENDING_READ_LOCK : PENDING_WRITE_LOCK,
@@ -218,21 +243,21 @@ bool push_blocking_lock_request( struct byte_range_lock *br_lck,
 	SMB_PERFCOUNT_DEFER_OP(&req->pcd, &req->pcd);
 	blr->req = talloc_move(blr, &req);
 
-	DLIST_ADD_END(blocking_lock_queue, blr, struct blocking_lock_record *);
-	recalc_brl_timeout();
+	DLIST_ADD_END(sconn->smb1.locks.blocking_lock_queue, blr, struct blocking_lock_record *);
+	recalc_brl_timeout(sconn);
 
 	/* Ensure we'll receive messages when this is unlocked. */
-	if (!blocking_lock_unlock_state) {
-		messaging_register(smbd_messaging_context(), NULL,
+	if (!sconn->smb1.locks.blocking_lock_unlock_state) {
+		messaging_register(sconn->msg_ctx, sconn,
 				   MSG_SMB_UNLOCK, received_unlock_msg);
-		blocking_lock_unlock_state = true;
+		sconn->smb1.locks.blocking_lock_unlock_state = true;
 	}
 
 	DEBUG(3,("push_blocking_lock_request: lock request blocked with "
-		"expiry time (%u sec. %u usec) (+%d msec) for fnum = %d, name = %s\n",
+		"expiry time (%u sec. %u usec) (+%d msec) for %s, name = %s\n",
 		(unsigned int)blr->expire_time.tv_sec,
 		(unsigned int)blr->expire_time.tv_usec, lock_timeout,
-		blr->fsp->fnum, fsp_str_dbg(blr->fsp)));
+		fsp_fnum_dbg(blr->fsp), fsp_str_dbg(blr->fsp)));
 
 	return True;
 }
@@ -243,7 +268,11 @@ bool push_blocking_lock_request( struct byte_range_lock *br_lck,
 
 static void reply_lockingX_success(struct blocking_lock_record *blr)
 {
-	reply_outbuf(blr->req, 2, 0);
+	struct smb_request *req = blr->req;
+
+	reply_outbuf(req, 2, 0);
+	SSVAL(req->outbuf, smb_vwv0, 0xff); /* andx chain ends */
+	SSVAL(req->outbuf, smb_vwv1, 0);    /* no andx offset */
 
 	/*
 	 * As this message is a lockingX call we must handle
@@ -253,8 +282,15 @@ static void reply_lockingX_success(struct blocking_lock_record *blr)
 	 * that here and must set up the chain info manually.
 	 */
 
-	chain_reply(blr->req);
-	TALLOC_FREE(blr->req->outbuf);
+	if (!srv_send_smb(req->sconn,
+			(char *)req->outbuf,
+			true, req->seqnum+1,
+			IS_CONN_ENCRYPTED(req->conn)||req->encrypted,
+			&req->pcd)) {
+		exit_server_cleanly("construct_reply: srv_send_smb failed.");
+	}
+
+	TALLOC_FREE(req->outbuf);
 }
 
 /****************************************************************************
@@ -274,9 +310,10 @@ static void generic_blocking_lock_error(struct blocking_lock_record *blr, NTSTAT
 		files_struct *fsp = blr->fsp;
 
 		if (fsp) {
-			fsp->last_lock_failure.context.smbpid = blr->lock_pid;
+			fsp->last_lock_failure.context.smblctx = blr->smblctx;
 			fsp->last_lock_failure.context.tid = fsp->conn->cnum;
-			fsp->last_lock_failure.context.pid = procid_self();
+			fsp->last_lock_failure.context.pid =
+				messaging_server_id(fsp->conn->sconn->msg_ctx);
 			fsp->last_lock_failure.start = blr->offset;
 			fsp->last_lock_failure.size = blr->count;
 			fsp->last_lock_failure.fnum = fsp->fnum;
@@ -286,7 +323,7 @@ static void generic_blocking_lock_error(struct blocking_lock_record *blr, NTSTAT
 	}
 
 	reply_nterror(blr->req, status);
-	if (!srv_send_smb(smbd_server_fd(), (char *)blr->req->outbuf,
+	if (!srv_send_smb(blr->req->sconn, (char *)blr->req->outbuf,
 			  true, blr->req->seqnum+1,
 			  blr->req->encrypted, NULL)) {
 		exit_server_cleanly("generic_blocking_lock_error: srv_send_smb failed.");
@@ -304,13 +341,13 @@ static void undo_locks_obtained(struct blocking_lock_record *blr)
 	files_struct *fsp = blr->fsp;
 	uint16 num_ulocks = SVAL(blr->req->vwv+6, 0);
 	uint64_t count = (uint64_t)0, offset = (uint64_t) 0;
-	uint32 lock_pid;
+	uint64_t smblctx;
 	unsigned char locktype = CVAL(blr->req->vwv+3, 0);
 	bool large_file_format = (locktype & LOCKING_ANDX_LARGE_FILES);
 	uint8_t *data;
 	int i;
 
-	data = (uint8_t *)blr->req->buf
+	data = discard_const_p(uint8_t, blr->req->buf)
 		+ ((large_file_format ? 20 : 10)*num_ulocks);
 
 	/* 
@@ -327,7 +364,7 @@ static void undo_locks_obtained(struct blocking_lock_record *blr)
 	for(i = blr->lock_num - 1; i >= 0; i--) {
 		bool err;
 
-		lock_pid = get_lock_pid( data, i, large_file_format);
+		smblctx = get_lock_pid( data, i, large_file_format);
 		count = get_lock_count( data, i, large_file_format);
 		offset = get_lock_offset( data, i, large_file_format, &err);
 
@@ -336,9 +373,9 @@ static void undo_locks_obtained(struct blocking_lock_record *blr)
 		 * request would never have been queued. JRA.
 		 */
 
-		do_unlock(smbd_messaging_context(),
+		do_unlock(fsp->conn->sconn->msg_ctx,
 			fsp,
-			lock_pid,
+			smblctx,
 			count,
 			offset,
 			WINDOWS_LOCK);
@@ -365,7 +402,7 @@ static void blocking_lock_reply_error(struct blocking_lock_record *blr, NTSTATUS
 			undo_locks_obtained(blr);
 		}
 		generic_blocking_lock_error(blr, status);
-                break;
+		break;
 	case SMBtrans2:
 	case SMBtranss2:
 		reply_nterror(blr->req, status);
@@ -376,7 +413,7 @@ static void blocking_lock_reply_error(struct blocking_lock_record *blr, NTSTATUS
 		 */
 		SCVAL(blr->req->outbuf,smb_com,SMBtrans2);
 
-		if (!srv_send_smb(smbd_server_fd(),
+		if (!srv_send_smb(blr->req->sconn,
 				  (char *)blr->req->outbuf,
 				  true, blr->req->seqnum+1,
 				  IS_CONN_ENCRYPTED(blr->fsp->conn),
@@ -404,12 +441,12 @@ static bool process_lockingX(struct blocking_lock_record *blr)
 	uint16 num_ulocks = SVAL(blr->req->vwv+6, 0);
 	uint16 num_locks = SVAL(blr->req->vwv+7, 0);
 	uint64_t count = (uint64_t)0, offset = (uint64_t)0;
-	uint32 lock_pid;
+	uint64_t smblctx;
 	bool large_file_format = (locktype & LOCKING_ANDX_LARGE_FILES);
 	uint8_t *data;
 	NTSTATUS status = NT_STATUS_OK;
 
-	data = (uint8_t *)blr->req->buf
+	data = discard_const_p(uint8_t, blr->req->buf)
 		+ ((large_file_format ? 20 : 10)*num_ulocks);
 
 	/* 
@@ -421,7 +458,7 @@ static bool process_lockingX(struct blocking_lock_record *blr)
 		struct byte_range_lock *br_lck = NULL;
 		bool err;
 
-		lock_pid = get_lock_pid( data, blr->lock_num, large_file_format);
+		smblctx = get_lock_pid( data, blr->lock_num, large_file_format);
 		count = get_lock_count( data, blr->lock_num, large_file_format);
 		offset = get_lock_offset( data, blr->lock_num, large_file_format, &err);
 
@@ -430,17 +467,17 @@ static bool process_lockingX(struct blocking_lock_record *blr)
 		 * request would never have been queued. JRA.
 		 */
 		errno = 0;
-		br_lck = do_lock(smbd_messaging_context(),
+		br_lck = do_lock(fsp->conn->sconn->msg_ctx,
 				fsp,
-				lock_pid,
+				smblctx,
 				count,
-				offset, 
+				offset,
 				((locktype & LOCKING_ANDX_SHARED_LOCK) ?
 					READ_LOCK : WRITE_LOCK),
 				WINDOWS_LOCK,
 				True,
 				&status,
-				&blr->blocking_pid,
+				&blr->blocking_smblctx,
 				blr);
 
 		TALLOC_FREE(br_lck);
@@ -455,8 +492,8 @@ static bool process_lockingX(struct blocking_lock_record *blr)
 		 * Success - we got all the locks.
 		 */
 
-		DEBUG(3,("process_lockingX file = %s, fnum=%d type=%d "
-			 "num_locks=%d\n", fsp_str_dbg(fsp), fsp->fnum,
+		DEBUG(3,("process_lockingX file = %s, %s, type=%d "
+			 "num_locks=%d\n", fsp_str_dbg(fsp), fsp_fnum_dbg(fsp),
 			 (unsigned int)locktype, num_locks));
 
 		reply_lockingX_success(blr);
@@ -478,9 +515,10 @@ static bool process_lockingX(struct blocking_lock_record *blr)
 	 * Still can't get all the locks - keep waiting.
 	 */
 
-	DEBUG(10,("process_lockingX: only got %d locks of %d needed for file %s, fnum = %d. \
-Waiting....\n", 
-		 blr->lock_num, num_locks, fsp_str_dbg(fsp), fsp->fnum));
+	DEBUG(10, ("process_lockingX: only got %d locks of %d needed for "
+		   "file %s, %s. Waiting....\n",
+		   blr->lock_num, num_locks, fsp_str_dbg(fsp),
+		   fsp_fnum_dbg(fsp)));
 
 	return False;
 }
@@ -494,16 +532,17 @@ static bool process_trans2(struct blocking_lock_record *blr)
 {
 	char params[2];
 	NTSTATUS status;
-	struct byte_range_lock *br_lck = do_lock(smbd_messaging_context(),
+	struct byte_range_lock *br_lck = do_lock(
+						blr->fsp->conn->sconn->msg_ctx,
 						blr->fsp,
-						blr->lock_pid,
+						blr->smblctx,
 						blr->count,
 						blr->offset,
 						blr->lock_type,
 						blr->lock_flav,
 						True,
 						&status,
-						&blr->blocking_pid,
+						&blr->blocking_smblctx,
 						blr);
 	TALLOC_FREE(br_lck);
 
@@ -524,7 +563,7 @@ static bool process_trans2(struct blocking_lock_record *blr)
 
 	SSVAL(params,0,0);
 	/* Fake up max_data_bytes here - we know it fits. */
-	send_trans2_replies(blr->fsp->conn, blr->req, params, 2, NULL, 0, 0xffff);
+	send_trans2_replies(blr->fsp->conn, blr->req, NT_STATUS_OK, params, 2, NULL, 0, 0xffff);
 	return True;
 }
 
@@ -551,13 +590,24 @@ static bool blocking_lock_record_process(struct blocking_lock_record *blr)
 
 /****************************************************************************
  Cancel entries by fnum from the blocking lock pending queue.
+ Called when a file is closed.
 *****************************************************************************/
 
-void cancel_pending_lock_requests_by_fid(files_struct *fsp, struct byte_range_lock *br_lck)
+void smbd_cancel_pending_lock_requests_by_fid(files_struct *fsp,
+					      struct byte_range_lock *br_lck,
+					      enum file_close_type close_type)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
 	struct blocking_lock_record *blr, *blr_cancelled, *next = NULL;
 
-	for(blr = blocking_lock_queue; blr; blr = next) {
+	if (sconn->using_smb2) {
+		cancel_pending_lock_requests_by_fid_smb2(fsp,
+					br_lck,
+					close_type);
+		return;
+	}
+
+	for(blr = sconn->smb1.locks.blocking_lock_queue; blr; blr = next) {
 		unsigned char locktype = 0;
 
 		next = blr->next;
@@ -570,11 +620,11 @@ void cancel_pending_lock_requests_by_fid(files_struct *fsp, struct byte_range_lo
 		}
 
 		DEBUG(10, ("remove_pending_lock_requests_by_fid - removing "
-			   "request type %d for file %s fnum = %d\n",
-			   blr->req->cmd, fsp_str_dbg(fsp), fsp->fnum));
+			   "request type %d for file %s, %s\n",
+			   blr->req->cmd, fsp_str_dbg(fsp), fsp_fnum_dbg(fsp)));
 
-		blr_cancelled = blocking_lock_cancel(fsp,
-				     blr->lock_pid,
+		blr_cancelled = blocking_lock_cancel_smb1(fsp,
+				     blr->smblctx,
 				     blr->offset,
 				     blr->count,
 				     blr->lock_flav,
@@ -584,8 +634,8 @@ void cancel_pending_lock_requests_by_fid(files_struct *fsp, struct byte_range_lo
 		SMB_ASSERT(blr_cancelled == blr);
 
 		brl_lock_cancel(br_lck,
-				blr->lock_pid,
-				procid_self(),
+				blr->smblctx,
+				messaging_server_id(sconn->msg_ctx),
 				blr->offset,
 				blr->count,
 				blr->lock_flav,
@@ -599,13 +649,15 @@ void cancel_pending_lock_requests_by_fid(files_struct *fsp, struct byte_range_lo
 
 /****************************************************************************
  Delete entries by mid from the blocking lock pending queue. Always send reply.
+ Only called from the SMB1 cancel code.
 *****************************************************************************/
 
-void remove_pending_lock_requests_by_mid(int mid)
+void remove_pending_lock_requests_by_mid_smb1(
+	struct smbd_server_connection *sconn, uint64_t mid)
 {
 	struct blocking_lock_record *blr, *next = NULL;
 
-	for(blr = blocking_lock_queue; blr; blr = next) {
+	for(blr = sconn->smb1.locks.blocking_lock_queue; blr; blr = next) {
 		files_struct *fsp;
 		struct byte_range_lock *br_lck;
 
@@ -619,14 +671,14 @@ void remove_pending_lock_requests_by_mid(int mid)
 		br_lck = brl_get_locks(talloc_tos(), fsp);
 
 		if (br_lck) {
-			DEBUG(10, ("remove_pending_lock_requests_by_mid - "
-				   "removing request type %d for file %s fnum "
-				   "= %d\n", blr->req->cmd, fsp_str_dbg(fsp),
-				   fsp->fnum ));
+			DEBUG(10, ("remove_pending_lock_requests_by_mid_smb1 - "
+				   "removing request type %d for file %s, %s\n",
+				   blr->req->cmd, fsp_str_dbg(fsp),
+				   fsp_fnum_dbg(fsp)));
 
 			brl_lock_cancel(br_lck,
-					blr->lock_pid,
-					procid_self(),
+					blr->smblctx,
+					messaging_server_id(sconn->msg_ctx),
 					blr->offset,
 					blr->count,
 					blr->lock_flav,
@@ -635,20 +687,22 @@ void remove_pending_lock_requests_by_mid(int mid)
 		}
 
 		blocking_lock_reply_error(blr,NT_STATUS_FILE_LOCK_CONFLICT);
-		DLIST_REMOVE(blocking_lock_queue, blr);
+		DLIST_REMOVE(sconn->smb1.locks.blocking_lock_queue, blr);
 		TALLOC_FREE(blr);
 	}
 }
 
 /****************************************************************************
  Is this mid a blocking lock request on the queue ?
+ Currently only called from the SMB1 unix extensions POSIX lock code.
 *****************************************************************************/
 
-bool blocking_lock_was_deferred(int mid)
+bool blocking_lock_was_deferred_smb1(
+	struct smbd_server_connection *sconn, uint64_t mid)
 {
 	struct blocking_lock_record *blr, *next = NULL;
 
-	for(blr = blocking_lock_queue; blr; blr = next) {
+	for(blr = sconn->smb1.locks.blocking_lock_queue; blr; blr = next) {
 		next = blr->next;
 		if(blr->req->mid == mid) {
 			return True;
@@ -667,24 +721,33 @@ static void received_unlock_msg(struct messaging_context *msg,
 				struct server_id server_id,
 				DATA_BLOB *data)
 {
+	struct smbd_server_connection *sconn =
+		talloc_get_type_abort(private_data,
+		struct smbd_server_connection);
+
 	DEBUG(10,("received_unlock_msg\n"));
-	process_blocking_lock_queue();
+	process_blocking_lock_queue(sconn);
 }
 
 /****************************************************************************
  Process the blocking lock queue. Note that this is only called as root.
 *****************************************************************************/
 
-void process_blocking_lock_queue(void)
+void process_blocking_lock_queue(struct smbd_server_connection *sconn)
 {
 	struct timeval tv_curr = timeval_current();
 	struct blocking_lock_record *blr, *next = NULL;
 
+	if (sconn->using_smb2) {
+		process_blocking_lock_queue_smb2(sconn, tv_curr);
+		return;
+	}
+
 	/*
 	 * Go through the queue and see if we can get any of the locks.
 	 */
 
-	for (blr = blocking_lock_queue; blr; blr = next) {
+	for (blr = sconn->smb1.locks.blocking_lock_queue; blr; blr = next) {
 
 		next = blr->next;
 
@@ -713,8 +776,8 @@ void process_blocking_lock_queue(void)
 
 			if (br_lck) {
 				brl_lock_cancel(br_lck,
-					blr->lock_pid,
-					procid_self(),
+					blr->smblctx,
+					messaging_server_id(sconn->msg_ctx),
 					blr->offset,
 					blr->count,
 					blr->lock_flav,
@@ -722,7 +785,7 @@ void process_blocking_lock_queue(void)
 				TALLOC_FREE(br_lck);
 			}
 
-			DLIST_REMOVE(blocking_lock_queue, blr);
+			DLIST_REMOVE(sconn->smb1.locks.blocking_lock_queue, blr);
 			TALLOC_FREE(blr);
 			continue;
 		}
@@ -745,13 +808,13 @@ void process_blocking_lock_queue(void)
 
 			if (br_lck) {
 				DEBUG(5,("process_blocking_lock_queue: "
-					 "pending lock fnum = %d for file %s "
-					 "timed out.\n", blr->fsp->fnum,
+					 "pending lock for %s, file %s "
+					 "timed out.\n", fsp_fnum_dbg(blr->fsp),
 					 fsp_str_dbg(blr->fsp)));
 
 				brl_lock_cancel(br_lck,
-					blr->lock_pid,
-					procid_self(),
+					blr->smblctx,
+					messaging_server_id(sconn->msg_ctx),
 					blr->offset,
 					blr->count,
 					blr->lock_flav,
@@ -760,12 +823,12 @@ void process_blocking_lock_queue(void)
 			}
 
 			blocking_lock_reply_error(blr,NT_STATUS_FILE_LOCK_CONFLICT);
-			DLIST_REMOVE(blocking_lock_queue, blr);
+			DLIST_REMOVE(sconn->smb1.locks.blocking_lock_queue, blr);
 			TALLOC_FREE(blr);
 		}
 	}
 
-	recalc_brl_timeout();
+	recalc_brl_timeout(sconn);
 }
 
 /****************************************************************************
@@ -783,6 +846,9 @@ static void process_blocking_lock_cancel_message(struct messaging_context *ctx,
 	NTSTATUS err;
 	const char *msg = (const char *)data->data;
 	struct blocking_lock_record *blr;
+	struct smbd_server_connection *sconn =
+		talloc_get_type_abort(private_data,
+		struct smbd_server_connection);
 
 	if (data->data == NULL) {
 		smb_panic("process_blocking_lock_cancel_message: null msg");
@@ -801,38 +867,40 @@ static void process_blocking_lock_cancel_message(struct messaging_context *ctx,
 		nt_errstr(err) ));
 
 	blocking_lock_reply_error(blr, err);
-	DLIST_REMOVE(blocking_lock_cancelled_queue, blr);
+	DLIST_REMOVE(sconn->smb1.locks.blocking_lock_cancelled_queue, blr);
 	TALLOC_FREE(blr);
 }
 
 /****************************************************************************
  Send ourselves a blocking lock cancelled message. Handled asynchronously above.
  Returns the blocking_lock_record that is being cancelled.
+ Only called from the SMB1 code.
 *****************************************************************************/
 
-struct blocking_lock_record *blocking_lock_cancel(files_struct *fsp,
-			uint32 lock_pid,
+struct blocking_lock_record *blocking_lock_cancel_smb1(files_struct *fsp,
+			uint64_t smblctx,
 			uint64_t offset,
 			uint64_t count,
 			enum brl_flavour lock_flav,
 			unsigned char locktype,
                         NTSTATUS err)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
 	char msg[MSG_BLOCKING_LOCK_CANCEL_SIZE];
 	struct blocking_lock_record *blr;
 
-	if (!blocking_lock_cancel_state) {
+	if (!sconn->smb1.locks.blocking_lock_cancel_state) {
 		/* Register our message. */
-		messaging_register(smbd_messaging_context(), NULL,
+		messaging_register(sconn->msg_ctx, sconn,
 				   MSG_SMB_BLOCKING_LOCK_CANCEL,
 				   process_blocking_lock_cancel_message);
 
-		blocking_lock_cancel_state = True;
+		sconn->smb1.locks.blocking_lock_cancel_state = True;
 	}
 
-	for (blr = blocking_lock_queue; blr; blr = blr->next) {
+	for (blr = sconn->smb1.locks.blocking_lock_queue; blr; blr = blr->next) {
 		if (fsp == blr->fsp &&
-				lock_pid == blr->lock_pid &&
+				smblctx == blr->smblctx &&
 				offset == blr->offset &&
 				count == blr->count &&
 				lock_flav == blr->lock_flav) {
@@ -852,14 +920,14 @@ struct blocking_lock_record *blocking_lock_cancel(files_struct *fsp,
 	}
 
 	/* Move to cancelled queue. */
-	DLIST_REMOVE(blocking_lock_queue, blr);
-	DLIST_ADD(blocking_lock_cancelled_queue, blr);
+	DLIST_REMOVE(sconn->smb1.locks.blocking_lock_queue, blr);
+	DLIST_ADD(sconn->smb1.locks.blocking_lock_cancelled_queue, blr);
 
 	/* Create the message. */
 	memcpy(msg, &blr, sizeof(blr));
 	memcpy(&msg[sizeof(blr)], &err, sizeof(NTSTATUS));
 
-	messaging_send_buf(smbd_messaging_context(), procid_self(),
+	messaging_send_buf(sconn->msg_ctx, messaging_server_id(sconn->msg_ctx),
 			   MSG_SMB_BLOCKING_LOCK_CANCEL,
 			   (uint8 *)&msg, sizeof(msg));
 
