@@ -4,17 +4,17 @@
    Copyright (C) Andrew Tridgell 1992-1998
    Copyright (C) Jeremy Allison 1998 - 2001
    Copyright (C) Volker Lendecke 2005
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -24,15 +24,7 @@
 #include "smbd/smbd.h"
 #include "smbd/globals.h"
 #include "messages.h"
-
-/****************************************************************************
- Get the number of current exclusive oplocks.
-****************************************************************************/
-
-int32 get_number_of_exclusive_open_oplocks(void)
-{
-  return exclusive_oplocks_open;
-}
+#include "../librpc/gen_ndr/open_files.h"
 
 /*
  * helper function used by the kernel oplock backends to post the break message
@@ -59,30 +51,35 @@ void break_kernel_oplock(struct messaging_context *msg_ctx, files_struct *fsp)
  if oplock set.
 ****************************************************************************/
 
-bool set_file_oplock(files_struct *fsp, int oplock_type)
+NTSTATUS set_file_oplock(files_struct *fsp, int oplock_type)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+	bool use_kernel = lp_kernel_oplocks(SNUM(fsp->conn)) && koplocks;
+
 	if (fsp->oplock_type == LEVEL_II_OPLOCK) {
-		if (koplocks &&
+		if (use_kernel &&
 		    !(koplocks->flags & KOPLOCKS_LEVEL2_SUPPORTED)) {
 			DEBUG(10, ("Refusing level2 oplock, kernel oplocks "
 				   "don't support them\n"));
-			return false;
+			return NT_STATUS_NOT_SUPPORTED;
 		}
 	}
 
 	if ((fsp->oplock_type != NO_OPLOCK) &&
 	    (fsp->oplock_type != FAKE_LEVEL_II_OPLOCK) &&
-	    koplocks &&
-	    !koplocks->ops->set_oplock(koplocks, fsp, oplock_type)) {
-		return False;
+	    use_kernel &&
+	    !koplocks->ops->set_oplock(koplocks, fsp, oplock_type))
+	{
+		return map_nt_error_from_unix(errno);
 	}
 
 	fsp->oplock_type = oplock_type;
 	fsp->sent_oplock_break = NO_BREAK_SENT;
 	if (oplock_type == LEVEL_II_OPLOCK) {
-		level_II_oplocks_open++;
+		sconn->oplocks.level_II_open++;
 	} else if (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type)) {
-		exclusive_oplocks_open++;
+		sconn->oplocks.exclusive_open++;
 	}
 
 	DEBUG(5,("set_file_oplock: granted oplock on file %s, %s/%lu, "
@@ -91,7 +88,7 @@ bool set_file_oplock(files_struct *fsp, int oplock_type)
 		 fsp->fh->gen_id, (int)fsp->open_time.tv_sec,
 		 (int)fsp->open_time.tv_usec ));
 
-	return True;
+	return NT_STATUS_OK;
 }
 
 /****************************************************************************
@@ -100,6 +97,9 @@ bool set_file_oplock(files_struct *fsp, int oplock_type)
 
 void release_file_oplock(files_struct *fsp)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+
 	if ((fsp->oplock_type != NO_OPLOCK) &&
 	    (fsp->oplock_type != FAKE_LEVEL_II_OPLOCK) &&
 	    koplocks) {
@@ -107,13 +107,13 @@ void release_file_oplock(files_struct *fsp)
 	}
 
 	if (fsp->oplock_type == LEVEL_II_OPLOCK) {
-		level_II_oplocks_open--;
+		sconn->oplocks.level_II_open--;
 	} else if (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type)) {
-		exclusive_oplocks_open--;
+		sconn->oplocks.exclusive_open--;
 	}
 
-	SMB_ASSERT(exclusive_oplocks_open>=0);
-	SMB_ASSERT(level_II_oplocks_open>=0);
+	SMB_ASSERT(sconn->oplocks.exclusive_open>=0);
+	SMB_ASSERT(sconn->oplocks.level_II_open>=0);
 
 	if (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type)) {
 		/* This doesn't matter for close. */
@@ -135,6 +135,9 @@ void release_file_oplock(files_struct *fsp)
 
 static void downgrade_file_oplock(files_struct *fsp)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+
 	if (!EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type)) {
 		DEBUG(0, ("trying to downgrade an already-downgraded oplock!\n"));
 		return;
@@ -144,8 +147,8 @@ static void downgrade_file_oplock(files_struct *fsp)
 		koplocks->ops->release_oplock(koplocks, fsp, LEVEL_II_OPLOCK);
 	}
 	fsp->oplock_type = LEVEL_II_OPLOCK;
-	exclusive_oplocks_open--;
-	level_II_oplocks_open++;
+	sconn->oplocks.exclusive_open--;
+	sconn->oplocks.level_II_open++;
 	fsp->sent_oplock_break = NO_BREAK_SENT;
 }
 
@@ -161,8 +164,7 @@ bool remove_oplock(files_struct *fsp)
 	struct share_mode_lock *lck;
 
 	/* Remove the oplock flag from the sharemode. */
-	lck = get_share_mode_lock(talloc_tos(), fsp->file_id, NULL, NULL,
-				  NULL);
+	lck = get_existing_share_mode_lock(talloc_tos(), fsp->file_id);
 	if (lck == NULL) {
 		DEBUG(0,("remove_oplock: failed to lock share entry for "
 			 "file %s\n", fsp_str_dbg(fsp)));
@@ -171,8 +173,8 @@ bool remove_oplock(files_struct *fsp)
 	ret = remove_share_oplock(lck, fsp);
 	if (!ret) {
 		DEBUG(0,("remove_oplock: failed to remove share oplock for "
-			 "file %s fnum %d, %s\n",
-			 fsp_str_dbg(fsp), fsp->fnum,
+			 "file %s, %s, %s\n",
+			 fsp_str_dbg(fsp), fsp_fnum_dbg(fsp),
 			 file_id_string_tos(&fsp->file_id)));
 	}
 	release_file_oplock(fsp);
@@ -188,8 +190,7 @@ bool downgrade_oplock(files_struct *fsp)
 	bool ret;
 	struct share_mode_lock *lck;
 
-	lck = get_share_mode_lock(talloc_tos(), fsp->file_id, NULL, NULL,
-				  NULL);
+	lck = get_existing_share_mode_lock(talloc_tos(), fsp->file_id);
 	if (lck == NULL) {
 		DEBUG(0,("downgrade_oplock: failed to lock share entry for "
 			 "file %s\n", fsp_str_dbg(fsp)));
@@ -198,8 +199,8 @@ bool downgrade_oplock(files_struct *fsp)
 	ret = downgrade_share_oplock(lck, fsp);
 	if (!ret) {
 		DEBUG(0,("downgrade_oplock: failed to downgrade share oplock "
-			 "for file %s fnum %d, file_id %s\n",
-			 fsp_str_dbg(fsp), fsp->fnum,
+			 "for file %s, %s, file_id %s\n",
+			 fsp_str_dbg(fsp), fsp_fnum_dbg(fsp),
 			 file_id_string_tos(&fsp->file_id)));
 	}
 
@@ -211,8 +212,9 @@ bool downgrade_oplock(files_struct *fsp)
 /*
  * Some kernel oplock implementations handle the notification themselves.
  */
-bool should_notify_deferred_opens()
+bool should_notify_deferred_opens(struct smbd_server_connection *sconn)
 {
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
 	return !(koplocks &&
 		(koplocks->flags & KOPLOCKS_DEFERRED_OPEN_NOTIFICATION));
 }
@@ -224,7 +226,7 @@ bool should_notify_deferred_opens()
 static char *new_break_message_smb1(TALLOC_CTX *mem_ctx,
 				   files_struct *fsp, int cmd)
 {
-	char *result = TALLOC_ARRAY(mem_ctx, char, smb_size + 8*2 + 0);
+	char *result = talloc_array(mem_ctx, char, smb_size + 8*2 + 0);
 
 	if (result == NULL) {
 		DEBUG(0, ("talloc failed\n"));
@@ -272,7 +274,8 @@ static files_struct *initial_break_processing(
 		dbgtext( "initial_break_processing: called for %s/%u\n",
 			 file_id_string_tos(&id), (int)file_id);
 		dbgtext( "Current oplocks_open (exclusive = %d, levelII = %d)\n",
-			exclusive_oplocks_open, level_II_oplocks_open );
+			sconn->oplocks.exclusive_open,
+			sconn->oplocks.level_II_open);
 	}
 
 	/*
@@ -287,7 +290,7 @@ static files_struct *initial_break_processing(
 		/* The file could have been closed in the meantime - return success. */
 		if( DEBUGLVL( 3 ) ) {
 			dbgtext( "initial_break_processing: cannot find open file with " );
-			dbgtext( "file_id %s gen_id = %lu", file_id_string_tos(&id), file_id);
+			dbgtext( "file_id %s gen_id = %lu, ", file_id_string_tos(&id), file_id);
 			dbgtext( "allowing break to succeed.\n" );
 		}
 		return NULL;
@@ -317,8 +320,8 @@ static files_struct *initial_break_processing(
 	return fsp;
 }
 
-static void oplock_timeout_handler(struct event_context *ctx,
-				   struct timed_event *te,
+static void oplock_timeout_handler(struct tevent_context *ctx,
+				   struct tevent_timer *te,
 				   struct timeval now,
 				   void *private_data)
 {
@@ -329,7 +332,6 @@ static void oplock_timeout_handler(struct event_context *ctx,
 	DEBUG(0, ("Oplock break failed for file %s -- replying anyway\n",
 		  fsp_str_dbg(fsp)));
 	remove_oplock(fsp);
-	reply_to_oplock_break_requests(fsp);
 }
 
 /*******************************************************************
@@ -338,6 +340,9 @@ static void oplock_timeout_handler(struct event_context *ctx,
 
 static void add_oplock_timeout_handler(files_struct *fsp)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+
 	/*
 	 * If kernel oplocks already notifies smbds when an oplock break times
 	 * out, just return.
@@ -353,9 +358,9 @@ static void add_oplock_timeout_handler(files_struct *fsp)
 	}
 
 	fsp->oplock_timeout =
-		event_add_timed(smbd_event_context(), fsp,
-				timeval_current_ofs(OPLOCK_BREAK_TIMEOUT, 0),
-				oplock_timeout_handler, fsp);
+		tevent_add_timer(fsp->conn->sconn->ev_ctx, fsp,
+				 timeval_current_ofs(OPLOCK_BREAK_TIMEOUT, 0),
+				 oplock_timeout_handler, fsp);
 
 	if (fsp->oplock_timeout == NULL) {
 		DEBUG(0, ("Could not add oplock timeout handler\n"));
@@ -407,7 +412,7 @@ void break_level2_to_none_async(files_struct *fsp)
 	SMB_ASSERT(fsp->oplock_type == LEVEL_II_OPLOCK);
 
 	DEBUG(10,("process_oplock_async_level2_break_message: sending break "
-		  "to none message for fid %d, file %s\n", fsp->fnum,
+		  "to none message for %s, file %s\n", fsp_fnum_dbg(fsp),
 		  fsp_str_dbg(fsp)));
 
 	/* Now send a break to none message to our client. */
@@ -429,27 +434,23 @@ void break_level2_to_none_async(files_struct *fsp)
  the client for LEVEL2.
 *******************************************************************/
 
-void process_oplock_async_level2_break_message(struct messaging_context *msg_ctx,
+static void process_oplock_async_level2_break_message(struct messaging_context *msg_ctx,
 						      void *private_data,
 						      uint32_t msg_type,
 						      struct server_id src,
 						      DATA_BLOB *data)
 {
-	struct smbd_server_connection *sconn;
 	struct share_mode_entry msg;
 	files_struct *fsp;
+	struct smbd_server_connection *sconn =
+		talloc_get_type_abort(private_data,
+		struct smbd_server_connection);
 
 	if (data->data == NULL) {
 		DEBUG(0, ("Got NULL buffer\n"));
 		return;
 	}
 
-	sconn = msg_ctx_to_sconn(msg_ctx);
-	if (sconn == NULL) {
-		DEBUG(1, ("could not find sconn\n"));
-		return;
-	}
-
 	if (data->length != MSG_SMB_SHARE_MODE_ENTRY_SIZE) {
 		DEBUG(0, ("Got invalid msg len %d\n", (int)data->length));
 		return;
@@ -459,8 +460,9 @@ void process_oplock_async_level2_break_message(struct messaging_context *msg_ctx
 	message_to_share_mode_entry(&msg, (char *)data->data);
 
 	DEBUG(10, ("Got oplock async level 2 break message from pid %s: "
-		   "%s/%lu\n", procid_str(talloc_tos(), &src),
-		   file_id_string_tos(&msg.id), msg.share_file_id));
+		   "%s/%llu\n", server_id_str(talloc_tos(), &src),
+		   file_id_string_tos(&msg.id),
+		   (unsigned long long)msg.share_file_id));
 
 	fsp = initial_break_processing(sconn, msg.id, msg.share_file_id);
 
@@ -485,22 +487,21 @@ static void process_oplock_break_message(struct messaging_context *msg_ctx,
 					 struct server_id src,
 					 DATA_BLOB *data)
 {
-	struct smbd_server_connection *sconn;
 	struct share_mode_entry msg;
 	files_struct *fsp;
 	bool break_to_level2 = False;
+	bool use_kernel;
+	struct smbd_server_connection *sconn =
+		talloc_get_type_abort(private_data,
+		struct smbd_server_connection);
+	struct server_id self = messaging_server_id(sconn->msg_ctx);
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
 
 	if (data->data == NULL) {
 		DEBUG(0, ("Got NULL buffer\n"));
 		return;
 	}
 
-	sconn = msg_ctx_to_sconn(msg_ctx);
-	if (sconn == NULL) {
-		DEBUG(1, ("could not find sconn\n"));
-		return;
-	}
-
 	if (data->length != MSG_SMB_SHARE_MODE_ENTRY_SIZE) {
 		DEBUG(0, ("Got invalid msg len %d\n", (int)data->length));
 		return;
@@ -509,32 +510,24 @@ static void process_oplock_break_message(struct messaging_context *msg_ctx,
 	/* De-linearize incoming message. */
 	message_to_share_mode_entry(&msg, (char *)data->data);
 
-	DEBUG(10, ("Got oplock break message from pid %s: %s/%lu\n",
-		   procid_str(talloc_tos(), &src), file_id_string_tos(&msg.id),
-		   msg.share_file_id));
+	DEBUG(10, ("Got oplock break message from pid %s: %s/%llu\n",
+		   server_id_str(talloc_tos(), &src),
+		   file_id_string_tos(&msg.id),
+		   (unsigned long long)msg.share_file_id));
 
 	fsp = initial_break_processing(sconn, msg.id, msg.share_file_id);
 
 	if (fsp == NULL) {
 		/* We hit a race here. Break messages are sent, and before we
-		 * get to process this message, we have closed the file. Reply
-		 * with 'ok, oplock broken' */
+		 * get to process this message, we have closed the file. */
 		DEBUG(3, ("Did not find fsp\n"));
-
-		/* We just send the same message back. */
-		messaging_send_buf(msg_ctx, src, MSG_SMB_BREAK_RESPONSE,
-				   (uint8 *)data->data,
-				   MSG_SMB_SHARE_MODE_ENTRY_SIZE);
 		return;
 	}
 
 	if (fsp->sent_oplock_break != NO_BREAK_SENT) {
-		/* Remember we have to inform the requesting PID when the
-		 * client replies */
-		msg.pid = src;
-		ADD_TO_ARRAY(NULL, struct share_mode_entry, msg,
-			     &fsp->pending_break_messages,
-			     &fsp->num_pending_break_messages);
+		/*
+		 * Nothing to do anymore
+		 */
 		return;
 	}
 
@@ -543,23 +536,21 @@ static void process_oplock_break_message(struct messaging_context *msg_ctx,
 		DEBUG(3, ("Already downgraded oplock on %s: %s\n",
 			  file_id_string_tos(&fsp->file_id),
 			  fsp_str_dbg(fsp)));
-		/* We just send the same message back. */
-		messaging_send_buf(msg_ctx, src, MSG_SMB_BREAK_RESPONSE,
-				   (uint8 *)data->data,
-				   MSG_SMB_SHARE_MODE_ENTRY_SIZE);
 		return;
 	}
 
-	if ((global_client_caps & CAP_LEVEL_II_OPLOCKS) && 
+	use_kernel = lp_kernel_oplocks(SNUM(fsp->conn)) && koplocks;
+
+	if ((global_client_caps & CAP_LEVEL_II_OPLOCKS) &&
 	    !(msg.op_type & FORCE_OPLOCK_BREAK_TO_NONE) &&
-	    !(koplocks && !(koplocks->flags & KOPLOCKS_LEVEL2_SUPPORTED)) &&
+	    !(use_kernel && !(koplocks->flags & KOPLOCKS_LEVEL2_SUPPORTED)) &&
 	    lp_level2_oplocks(SNUM(fsp->conn))) {
 		break_to_level2 = True;
 	}
 
 	/* Need to wait before sending a break
 	   message if we sent ourselves this message. */
-	if (procid_is_me(&src)) {
+	if (serverid_equal(&self, &src)) {
 		wait_before_sending_break();
 	}
 
@@ -573,11 +564,6 @@ static void process_oplock_break_message(struct messaging_context *msg_ctx,
 
 	fsp->sent_oplock_break = break_to_level2 ? LEVEL_II_BREAK_SENT:BREAK_TO_NONE_SENT;
 
-	msg.pid = src;
-	ADD_TO_ARRAY(NULL, struct share_mode_entry, msg,
-		     &fsp->pending_break_messages,
-		     &fsp->num_pending_break_messages);
-
 	add_oplock_timeout_handler(fsp);
 }
 
@@ -591,10 +577,12 @@ static void process_kernel_oplock_break(struct messaging_context *msg_ctx,
 					struct server_id src,
 					DATA_BLOB *data)
 {
-	struct smbd_server_connection *sconn;
 	struct file_id id;
 	unsigned long file_id;
 	files_struct *fsp;
+	struct smbd_server_connection *sconn =
+		talloc_get_type_abort(private_data,
+		struct smbd_server_connection);
 
 	if (data->data == NULL) {
 		DEBUG(0, ("Got NULL buffer\n"));
@@ -606,18 +594,12 @@ static void process_kernel_oplock_break(struct messaging_context *msg_ctx,
 		return;
 	}
 
-	sconn = msg_ctx_to_sconn(msg_ctx);
-	if (sconn == NULL) {
-		DEBUG(1, ("could not find sconn\n"));
-		return;
-	}
-
 	/* Pull the data from the message. */
 	pull_file_id_24((char *)data->data, &id);
 	file_id = (unsigned long)IVAL(data->data, 24);
 
 	DEBUG(10, ("Got kernel oplock break message from pid %s: %s/%u\n",
-		   procid_str(talloc_tos(), &src), file_id_string_tos(&id),
+		   server_id_str(talloc_tos(), &src), file_id_string_tos(&id),
 		   (unsigned int)file_id));
 
 	fsp = initial_break_processing(sconn, id, file_id);
@@ -646,97 +628,11 @@ static void process_kernel_oplock_break(struct messaging_context *msg_ctx,
 	add_oplock_timeout_handler(fsp);
 }
 
-void reply_to_oplock_break_requests(files_struct *fsp)
-{
-	int i;
-
-	/*
-	 * If kernel oplocks already notifies smbds when oplocks are
-	 * broken/removed, just return.
-	 */
-	if (koplocks &&
-	    (koplocks->flags & KOPLOCKS_OPLOCK_BROKEN_NOTIFICATION)) {
-		return;
-	}
-
-	for (i=0; i<fsp->num_pending_break_messages; i++) {
-		struct share_mode_entry *e = &fsp->pending_break_messages[i];
-		char msg[MSG_SMB_SHARE_MODE_ENTRY_SIZE];
-
-		share_mode_entry_to_message(msg, e);
-
-		messaging_send_buf(fsp->conn->sconn->msg_ctx, e->pid,
-				   MSG_SMB_BREAK_RESPONSE,
-				   (uint8 *)msg,
-				   MSG_SMB_SHARE_MODE_ENTRY_SIZE);
-	}
-
-	SAFE_FREE(fsp->pending_break_messages);
-	fsp->num_pending_break_messages = 0;
-	if (fsp->oplock_timeout != NULL) {
-		/* Remove the timed event handler. */
-		TALLOC_FREE(fsp->oplock_timeout);
-		fsp->oplock_timeout = NULL;
-	}
-	return;
-}
-
-static void process_oplock_break_response(struct messaging_context *msg_ctx,
-					  void *private_data,
-					  uint32_t msg_type,
-					  struct server_id src,
-					  DATA_BLOB *data)
-{
-	struct share_mode_entry msg;
-
-	if (data->data == NULL) {
-		DEBUG(0, ("Got NULL buffer\n"));
-		return;
-	}
-
-	if (data->length != MSG_SMB_SHARE_MODE_ENTRY_SIZE) {
-		DEBUG(0, ("Got invalid msg len %u\n",
-			  (unsigned int)data->length));
-		return;
-	}
-
-	/* De-linearize incoming message. */
-	message_to_share_mode_entry(&msg, (char *)data->data);
-
-	DEBUG(10, ("Got oplock break response from pid %s: %s/%lu mid %llu\n",
-		   procid_str(talloc_tos(), &src), file_id_string_tos(&msg.id),
-		   msg.share_file_id, (unsigned long long)msg.op_mid));
-
-	schedule_deferred_open_message_smb(msg.op_mid);
-}
-
-static void process_open_retry_message(struct messaging_context *msg_ctx,
-				       void *private_data,
-				       uint32_t msg_type,
-				       struct server_id src,
-				       DATA_BLOB *data)
-{
-	struct share_mode_entry msg;
-	
-	if (data->data == NULL) {
-		DEBUG(0, ("Got NULL buffer\n"));
-		return;
-	}
-
-	if (data->length != MSG_SMB_SHARE_MODE_ENTRY_SIZE) {
-		DEBUG(0, ("Got invalid msg len %d\n", (int)data->length));
-		return;
-	}
-
-	/* De-linearize incoming message. */
-	message_to_share_mode_entry(&msg, (char *)data->data);
-
-	DEBUG(10, ("Got open retry msg from pid %s: %s mid %llu\n",
-		   procid_str(talloc_tos(), &src), file_id_string_tos(&msg.id),
-		   (unsigned long long)msg.op_mid));
-
-	schedule_deferred_open_message_smb(msg.op_mid);
-}
+struct break_to_none_state {
+	struct smbd_server_connection *sconn;
+	struct file_id id;
+};
+static void do_break_to_none(struct tevent_req *req);
 
 /****************************************************************************
  This function is called on any file modification or lock request. If a file
@@ -747,8 +643,9 @@ static void process_open_retry_message(struct messaging_context *msg_ctx,
 static void contend_level2_oplocks_begin_default(files_struct *fsp,
 					      enum level2_contention_type type)
 {
-	int i;
-	struct share_mode_lock *lck;
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct tevent_req *req;
+	struct break_to_none_state *state;
 
 	/*
 	 * If this file is level II oplocked then we need
@@ -761,19 +658,59 @@ static void contend_level2_oplocks_begin_default(files_struct *fsp,
 	if (!LEVEL_II_OPLOCK_TYPE(fsp->oplock_type))
 		return;
 
-	lck = get_share_mode_lock(talloc_tos(), fsp->file_id, NULL, NULL,
-				  NULL);
-	if (lck == NULL) {
-		DEBUG(0,("release_level_2_oplocks_on_change: failed to lock "
-			 "share mode entry for file %s.\n", fsp_str_dbg(fsp)));
+	/*
+	 * When we get here we might have a brlock entry locked. Also
+	 * locking the share mode entry would violate the locking
+	 * order. Breaking level2 oplocks to none is asynchronous
+	 * anyway, so we postpone this into an immediate timed event.
+	 */
+
+	state = talloc(sconn, struct break_to_none_state);
+	if (state == NULL) {
+		DEBUG(1, ("talloc failed\n"));
 		return;
 	}
+	state->sconn = sconn;
+	state->id = fsp->file_id;
+
+	req = tevent_wakeup_send(state, sconn->ev_ctx, timeval_set(0, 0));
+	if (req == NULL) {
+		DEBUG(1, ("tevent_wakeup_send failed\n"));
+		TALLOC_FREE(state);
+		return;
+	}
+	tevent_req_set_callback(req, do_break_to_none, state);
+	return;
+}
+
+static void do_break_to_none(struct tevent_req *req)
+{
+	struct break_to_none_state *state = tevent_req_callback_data(
+		req, struct break_to_none_state);
+	struct server_id self = messaging_server_id(state->sconn->msg_ctx);
+	bool ret;
+	int i;
+	struct share_mode_lock *lck;
+
+	ret = tevent_wakeup_recv(req);
+	TALLOC_FREE(req);
+	if (!ret) {
+		DEBUG(1, ("tevent_wakeup_recv failed\n"));
+		goto done;
+	}
+	lck = get_existing_share_mode_lock(talloc_tos(), state->id);
+	if (lck == NULL) {
+		DEBUG(1, ("release_level_2_oplocks_on_change: failed to lock "
+			  "share mode entry for file %s.\n",
+			  file_id_string_tos(&state->id)));
+		goto done;
+	}
 
 	DEBUG(10,("release_level_2_oplocks_on_change: num_share_modes = %d\n", 
-		  lck->num_share_modes ));
+		  lck->data->num_share_modes ));
 
-	for(i = 0; i < lck->num_share_modes; i++) {
-		struct share_mode_entry *share_entry = &lck->share_modes[i];
+	for(i = 0; i < lck->data->num_share_modes; i++) {
+		struct share_mode_entry *share_entry = &lck->data->share_modes[i];
 		char msg[MSG_SMB_SHARE_MODE_ENTRY_SIZE];
 
 		if (!is_valid_share_mode_entry(share_entry)) {
@@ -822,9 +759,9 @@ static void contend_level2_oplocks_begin_default(files_struct *fsp,
  		 * Bugid #5980.
  		 */
 
-		if (procid_is_me(&share_entry->pid)) {
+		if (serverid_equal(&self, &share_entry->pid)) {
 			struct files_struct *cur_fsp =
-				initial_break_processing(fsp->conn->sconn,
+				initial_break_processing(state->sconn,
 					share_entry->id,
 					share_entry->share_file_id);
 			wait_before_sending_break();
@@ -835,7 +772,7 @@ static void contend_level2_oplocks_begin_default(files_struct *fsp,
 				"Did not find fsp, ignoring\n"));
 			}
 		} else {
-			messaging_send_buf(fsp->conn->sconn->msg_ctx,
+			messaging_send_buf(state->sconn->msg_ctx,
 					share_entry->pid,
 					MSG_SMB_ASYNC_LEVEL2_BREAK,
 					(uint8 *)msg,
@@ -847,11 +784,17 @@ static void contend_level2_oplocks_begin_default(files_struct *fsp,
 	   in the share mode lock db. */
 
 	TALLOC_FREE(lck);
+done:
+	TALLOC_FREE(state);
+	return;
 }
 
-void contend_level2_oplocks_begin(files_struct *fsp,
+void smbd_contend_level2_oplocks_begin(files_struct *fsp,
 				  enum level2_contention_type type)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+
 	if (koplocks && koplocks->ops->contend_level2_oplocks_begin) {
 		koplocks->ops->contend_level2_oplocks_begin(fsp, type);
 		return;
@@ -860,9 +803,12 @@ void contend_level2_oplocks_begin(files_struct *fsp,
 	contend_level2_oplocks_begin_default(fsp, type);
 }
 
-void contend_level2_oplocks_end(files_struct *fsp,
+void smbd_contend_level2_oplocks_end(files_struct *fsp,
 				enum level2_contention_type type)
 {
+	struct smbd_server_connection *sconn = fsp->conn->sconn;
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+
 	/* Only kernel oplocks implement this so far */
 	if (koplocks && koplocks->ops->contend_level2_oplocks_end) {
 		koplocks->ops->contend_level2_oplocks_end(fsp, type);
@@ -917,31 +863,30 @@ void message_to_share_mode_entry(struct share_mode_entry *e, char *msg)
  Setup oplocks for this process.
 ****************************************************************************/
 
-bool init_oplocks(struct messaging_context *msg_ctx)
+bool init_oplocks(struct smbd_server_connection *sconn)
 {
 	DEBUG(3,("init_oplocks: initializing messages.\n"));
 
-	messaging_register(msg_ctx, NULL, MSG_SMB_BREAK_REQUEST,
+	messaging_register(sconn->msg_ctx, sconn, MSG_SMB_BREAK_REQUEST,
 			   process_oplock_break_message);
-	messaging_register(msg_ctx, NULL, MSG_SMB_ASYNC_LEVEL2_BREAK,
+	messaging_register(sconn->msg_ctx, sconn, MSG_SMB_ASYNC_LEVEL2_BREAK,
 			   process_oplock_async_level2_break_message);
-	messaging_register(msg_ctx, NULL, MSG_SMB_BREAK_RESPONSE,
-			   process_oplock_break_response);
-	messaging_register(msg_ctx, NULL, MSG_SMB_KERNEL_BREAK,
+	messaging_register(sconn->msg_ctx, sconn, MSG_SMB_KERNEL_BREAK,
 			   process_kernel_oplock_break);
-	messaging_register(msg_ctx, NULL, MSG_SMB_OPEN_RETRY,
-			   process_open_retry_message);
+	return true;
+}
 
-	if (lp_kernel_oplocks()) {
+void init_kernel_oplocks(struct smbd_server_connection *sconn)
+{
+	struct kernel_oplocks *koplocks = sconn->oplocks.kernel_ops;
+
+	/* only initialize once */
+	if (koplocks == NULL) {
 #if HAVE_KERNEL_OPLOCKS_IRIX
-		koplocks = irix_init_kernel_oplocks(NULL);
+		koplocks = irix_init_kernel_oplocks(sconn);
 #elif HAVE_KERNEL_OPLOCKS_LINUX
-		koplocks = linux_init_kernel_oplocks(NULL);
-#elif HAVE_ONEFS
-#error Isilon, please check if the NULL context is okay here. Thanks!
-		koplocks = onefs_init_kernel_oplocks(NULL);
+		koplocks = linux_init_kernel_oplocks(sconn);
 #endif
+		sconn->oplocks.kernel_ops = koplocks;
 	}
-
-	return True;
 }
