@@ -19,7 +19,8 @@
 
 #include "includes.h"
 #include "winbindd.h"
-#include "librpc/gen_ndr/cli_wbint.h"
+#include "librpc/gen_ndr/ndr_wbint_c.h"
+#include "../libcli/security/security.h"
 
 struct wb_queryuser_state {
 	struct dom_sid sid;
@@ -53,7 +54,7 @@ struct tevent_req *wb_queryuser_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
-	subreq = rpccli_wbint_QueryUser_send(state, ev, domain->child.rpccli,
+	subreq = dcerpc_wbint_QueryUser_send(state, ev, dom_child_handle(domain),
 					     &state->sid, state->info);
 	if (tevent_req_nomem(subreq, req)) {
 		return tevent_req_post(req, ev);
@@ -70,16 +71,12 @@ static void wb_queryuser_done(struct tevent_req *subreq)
 		req, struct wb_queryuser_state);
 	NTSTATUS status, result;
 
-	status = rpccli_wbint_QueryUser_recv(subreq, state->info, &result);
+	status = dcerpc_wbint_QueryUser_recv(subreq, state->info, &result);
 	TALLOC_FREE(subreq);
-	if (!NT_STATUS_IS_OK(status)) {
+	if (any_nt_status_not_ok(status, result, &status)) {
 		tevent_req_nterror(req, status);
 		return;
 	}
-	if (!NT_STATUS_IS_OK(result)) {
-		tevent_req_nterror(req, result);
-		return;
-	}
 	tevent_req_done(req);
 }
 
