@@ -29,7 +29,14 @@
 
 #include "includes.h"
 #include "winbindd.h"
-#include "../../nsswitch/libwbclient/wbc_async.h"
+#include "nsswitch/wb_reqtrans.h"
+#include "secrets.h"
+#include "../lib/util/select.h"
+#include "../libcli/security/security.h"
+#include "system/select.h"
+#include "messages.h"
+#include "../lib/util/tevent_unix.h"
+#include "lib/param/loadparm.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_WINBIND
@@ -37,7 +44,7 @@
 extern bool override_logfile;
 extern struct winbindd_methods cache_methods;
 
-static struct winbindd_child *children = NULL;
+static struct winbindd_child *winbindd_children = NULL;
 
 /* Read some data from a client connection */
 
@@ -123,7 +130,7 @@ struct tevent_req *wb_child_request_send(TALLOC_CTX *mem_ctx,
 
 	if (!tevent_queue_add(child->queue, ev, req,
 			      wb_child_request_trigger, NULL)) {
-		tevent_req_nomem(NULL, req);
+		tevent_req_oom(req);
 		return tevent_req_post(req, ev);
 	}
 	return req;
@@ -147,12 +154,7 @@ static void wb_child_request_trigger(struct tevent_req *req,
 		return;
 	}
 	tevent_req_set_callback(subreq, wb_child_request_done, req);
-
-	if (!tevent_req_set_endtime(req, state->ev,
-				    timeval_current_ofs(300, 0))) {
-		tevent_req_nomem(NULL, req);
-                return;
-        }
+	tevent_req_set_endtime(req, state->ev, timeval_current_ofs(300, 0));
 }
 
 static void wb_child_request_done(struct tevent_req *subreq)
@@ -172,7 +174,7 @@ static void wb_child_request_done(struct tevent_req *subreq)
 		 */
 		close(state->child->sock);
 		state->child->sock = -1;
-		DLIST_REMOVE(children, state->child);
+		DLIST_REMOVE(winbindd_children, state->child);
 		tevent_req_error(req, err);
 		return;
 	}
@@ -192,9 +194,47 @@ int wb_child_request_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 	return 0;
 }
 
+static bool winbindd_child_busy(struct winbindd_child *child)
+{
+	return tevent_queue_length(child->queue) > 0;
+}
+
+static struct winbindd_child *find_idle_child(struct winbindd_domain *domain)
+{
+	int i;
+
+	for (i=0; i<lp_winbind_max_domain_connections(); i++) {
+		if (!winbindd_child_busy(&domain->children[i])) {
+			return &domain->children[i];
+		}
+	}
+
+	return NULL;
+}
+
+struct winbindd_child *choose_domain_child(struct winbindd_domain *domain)
+{
+	struct winbindd_child *result;
+
+	result = find_idle_child(domain);
+	if (result != NULL) {
+		return result;
+	}
+	return &domain->children[rand() % lp_winbind_max_domain_connections()];
+}
+
+struct dcerpc_binding_handle *dom_child_handle(struct winbindd_domain *domain)
+{
+	struct winbindd_child *child;
+
+	child = choose_domain_child(domain);
+	return child->binding_handle;
+}
+
 struct wb_domain_request_state {
 	struct tevent_context *ev;
 	struct winbindd_domain *domain;
+	struct winbindd_child *child;
 	struct winbindd_request *request;
 	struct winbindd_request *init_req;
 	struct winbindd_response *response;
@@ -218,8 +258,10 @@ struct tevent_req *wb_domain_request_send(TALLOC_CTX *mem_ctx,
 		return NULL;
 	}
 
+	state->child = choose_domain_child(domain);
+
 	if (domain->initialized) {
-		subreq = wb_child_request_send(state, ev, &domain->child,
+		subreq = wb_child_request_send(state, ev, state->child,
 					       request);
 		if (tevent_req_nomem(subreq, req)) {
 			return tevent_req_post(req, ev);
@@ -241,11 +283,10 @@ struct tevent_req *wb_domain_request_send(TALLOC_CTX *mem_ctx,
 		/* The primary domain has to find the DC name itself */
 		state->init_req->cmd = WINBINDD_INIT_CONNECTION;
 		fstrcpy(state->init_req->domain_name, domain->name);
-		state->init_req->data.init_conn.is_primary =
-			domain->primary ? true : false;
+		state->init_req->data.init_conn.is_primary = domain->primary;
 		fstrcpy(state->init_req->data.init_conn.dcname, "");
 
-		subreq = wb_child_request_send(state, ev, &domain->child,
+		subreq = wb_child_request_send(state, ev, state->child,
 					       state->init_req);
 		if (tevent_req_nomem(subreq, req)) {
 			return tevent_req_post(req, ev);
@@ -266,7 +307,7 @@ struct tevent_req *wb_domain_request_send(TALLOC_CTX *mem_ctx,
 	state->init_req->cmd = WINBINDD_GETDCNAME;
 	fstrcpy(state->init_req->domain_name, domain->name);
 
-	subreq = wb_child_request_send(state, ev, &domain->child, request);
+	subreq = wb_child_request_send(state, ev, state->child, request);
 	if (tevent_req_nomem(subreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -297,7 +338,7 @@ static void wb_domain_request_gotdc(struct tevent_req *subreq)
 
 	TALLOC_FREE(response);
 
-	subreq = wb_child_request_send(state, state->ev, &state->domain->child,
+	subreq = wb_child_request_send(state, state->ev, state->child,
 				       state->init_req);
 	if (tevent_req_nomem(subreq, req)) {
 		return;
@@ -328,8 +369,26 @@ static void wb_domain_request_initialized(struct tevent_req *subreq)
 		tevent_req_error(req, EINVAL);
 		return;
 	}
-	fstrcpy(state->domain->name, response->data.domain_info.name);
-	fstrcpy(state->domain->alt_name, response->data.domain_info.alt_name);
+
+	talloc_free(state->domain->name);
+	state->domain->name = talloc_strdup(state->domain,
+					    response->data.domain_info.name);
+	if (state->domain->name == NULL) {
+		tevent_req_error(req, ENOMEM);
+		return;
+	}
+
+	if (response->data.domain_info.alt_name[0] != '\0') {
+		talloc_free(state->domain->alt_name);
+
+		state->domain->alt_name = talloc_strdup(state->domain,
+				response->data.domain_info.alt_name);
+		if (state->domain->alt_name == NULL) {
+			tevent_req_error(req, ENOMEM);
+			return;
+		}
+	}
+
 	state->domain->native_mode = response->data.domain_info.native_mode;
 	state->domain->active_directory =
 		response->data.domain_info.active_directory;
@@ -337,7 +396,7 @@ static void wb_domain_request_initialized(struct tevent_req *subreq)
 
 	TALLOC_FREE(response);
 
-	subreq = wb_child_request_send(state, state->ev, &state->domain->child,
+	subreq = wb_child_request_send(state, state->ev, state->child,
 				       state->request);
 	if (tevent_req_nomem(subreq, req)) {
 		return;
@@ -376,95 +435,6 @@ int wb_domain_request_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 	return 0;
 }
 
-struct domain_request_state {
-	struct winbindd_domain *domain;
-	struct winbindd_request *request;
-	struct winbindd_response *response;
-	void (*continuation)(void *private_data_data, bool success);
-	void *private_data_data;
-};
-
-static void async_domain_request_done(struct tevent_req *req);
-
-void async_domain_request(TALLOC_CTX *mem_ctx,
-			  struct winbindd_domain *domain,
-			  struct winbindd_request *request,
-			  struct winbindd_response *response,
-			  void (*continuation)(void *private_data_data, bool success),
-			  void *private_data_data)
-{
-	struct tevent_req *subreq;
-	struct domain_request_state *state;
-
-	state = TALLOC_P(mem_ctx, struct domain_request_state);
-	if (state == NULL) {
-		DEBUG(0, ("talloc failed\n"));
-		continuation(private_data_data, False);
-		return;
-	}
-
-	state->domain = domain;
-	state->request = request;
-	state->response = response;
-	state->continuation = continuation;
-	state->private_data_data = private_data_data;
-
-	subreq = wb_domain_request_send(state, winbind_event_context(),
-					domain, request);
-	if (subreq == NULL) {
-		DEBUG(5, ("wb_domain_request_send failed\n"));
-		continuation(private_data_data, false);
-		return;
-	}
-	tevent_req_set_callback(subreq, async_domain_request_done, state);
-}
-
-static void async_domain_request_done(struct tevent_req *req)
-{
-	struct domain_request_state *state = tevent_req_callback_data(
-		req, struct domain_request_state);
-	struct winbindd_response *response;
-	int ret, err;
-
-	ret = wb_domain_request_recv(req, state, &response, &err);
-	TALLOC_FREE(req);
-	if (ret == -1) {
-		DEBUG(5, ("wb_domain_request returned %s\n", strerror(err)));
-		state->continuation(state->private_data_data, false);
-		return;
-	}
-	*(state->response) = *response;
-	state->continuation(state->private_data_data, true);
-}
-
-static void recvfrom_child(void *private_data_data, bool success)
-{
-	struct winbindd_cli_state *state =
-		talloc_get_type_abort(private_data_data, struct winbindd_cli_state);
-	enum winbindd_result result = state->response->result;
-
-	/* This is an optimization: The child has written directly to the
-	 * response buffer. The request itself is still in pending state,
-	 * state that in the result code. */
-
-	state->response->result = WINBINDD_PENDING;
-
-	if ((!success) || (result != WINBINDD_OK)) {
-		request_error(state);
-		return;
-	}
-
-	request_ok(state);
-}
-
-void sendto_domain(struct winbindd_cli_state *state,
-		   struct winbindd_domain *domain)
-{
-	async_domain_request(state->mem_ctx, domain,
-			     state->request, state->response,
-			     recvfrom_child, state);
-}
-
 static void child_process_request(struct winbindd_child *child,
 				  struct winbindd_cli_state *state)
 {
@@ -491,7 +461,7 @@ static void child_process_request(struct winbindd_child *child,
 		}
 	}
 
-	DEBUG(1 ,("child_process_request: unknown request fn number %d\n",
+	DEBUG(1, ("child_process_request: unknown request fn number %d\n",
 		  (int)state->request->cmd));
 	state->response->result = WINBINDD_ERROR;
 }
@@ -502,10 +472,31 @@ void setup_child(struct winbindd_domain *domain, struct winbindd_child *child,
 		 const char *logname)
 {
 	if (logprefix && logname) {
+		char *logbase = NULL;
+
+		if (*lp_logfile(talloc_tos())) {
+			char *end = NULL;
+
+			if (asprintf(&logbase, "%s", lp_logfile(talloc_tos())) < 0) {
+				smb_panic("Internal error: asprintf failed");
+			}
+
+			if ((end = strrchr_m(logbase, '/'))) {
+				*end = '\0';
+			}
+		} else {
+			if (asprintf(&logbase, "%s", get_dyn_LOGFILEBASE()) < 0) {
+				smb_panic("Internal error: asprintf failed");
+			}
+		}
+
 		if (asprintf(&child->logfilename, "%s/%s-%s",
-			     get_dyn_LOGFILEBASE(), logprefix, logname) < 0) {
+			     logbase, logprefix, logname) < 0) {
+			SAFE_FREE(logbase);
 			smb_panic("Internal error: asprintf failed");
 		}
+
+		SAFE_FREE(logbase);
 	} else {
 		smb_panic("Internal error: logprefix == NULL && "
 			  "logname == NULL");
@@ -516,15 +507,15 @@ void setup_child(struct winbindd_domain *domain, struct winbindd_child *child,
 	child->table = table;
 	child->queue = tevent_queue_create(NULL, "winbind_child");
 	SMB_ASSERT(child->queue != NULL);
-	child->rpccli = wbint_rpccli_create(NULL, domain, child);
-	SMB_ASSERT(child->rpccli != NULL);
+	child->binding_handle = wbint_binding_handle(NULL, domain, child);
+	SMB_ASSERT(child->binding_handle != NULL);
 }
 
 void winbind_child_died(pid_t pid)
 {
 	struct winbindd_child *child;
 
-	for (child = children; child != NULL; child = child->next) {
+	for (child = winbindd_children; child != NULL; child = child->next) {
 		if (child->pid == pid) {
 			break;
 		}
@@ -537,8 +528,13 @@ void winbind_child_died(pid_t pid)
 
 	/* This will be re-added in fork_domain_child() */
 
-	DLIST_REMOVE(children, child);
+	DLIST_REMOVE(winbindd_children, child);
 	child->pid = 0;
+
+	if (child->sock != -1) {
+		close(child->sock);
+		child->sock = -1;
+	}
 }
 
 /* Ensure any negative cache entries with the netbios or realm names are removed. */
@@ -546,7 +542,7 @@ void winbind_child_died(pid_t pid)
 void winbindd_flush_negative_conn_cache(struct winbindd_domain *domain)
 {
 	flush_negative_conn_cache_for_domain(domain->name);
-	if (*domain->alt_name) {
+	if (domain->alt_name != NULL) {
 		flush_negative_conn_cache_for_domain(domain->alt_name);
 	}
 }
@@ -569,7 +565,7 @@ void winbind_msg_debug(struct messaging_context *msg_ctx,
 
 	debug_message(msg_ctx, private_data, MSG_DEBUG, server_id, data);
 
-	for (child = children; child != NULL; child = child->next) {
+	for (child = winbindd_children; child != NULL; child = child->next) {
 
 		DEBUG(10,("winbind_msg_debug: sending message to pid %u.\n",
 			(unsigned int)child->pid));
@@ -614,8 +610,8 @@ void winbind_msg_offline(struct messaging_context *msg_ctx,
 		set_domain_offline(domain);
 	}
 
-	for (child = children; child != NULL; child = child->next) {
-		/* Don't send message to internal childs.  We've already
+	for (child = winbindd_children; child != NULL; child = child->next) {
+		/* Don't send message to internal children.  We've already
 		   done so above. */
 		if (!child->domain || winbindd_internal_child(child)) {
 			continue;
@@ -634,7 +630,7 @@ void winbind_msg_offline(struct messaging_context *msg_ctx,
 
 		messaging_send_buf(msg_ctx, pid_to_procid(child->pid),
 				   MSG_WINBIND_OFFLINE,
-				   (uint8 *)child->domain->name,
+				   (const uint8_t *)child->domain->name,
 				   strlen(child->domain->name)+1);
 	}
 }
@@ -683,13 +679,13 @@ void winbind_msg_online(struct messaging_context *msg_ctx,
 				messaging_send_buf(msg_ctx,
 						   pid_to_procid(idmap->pid), 
 						   MSG_WINBIND_ONLINE,
-						   (uint8 *)domain->name,
+						   (const uint8_t *)domain->name,
 						   strlen(domain->name)+1);
 			}
 		}
 	}
 
-	for (child = children; child != NULL; child = child->next) {
+	for (child = winbindd_children; child != NULL; child = child->next) {
 		/* Don't send message to internal childs. */
 		if (!child->domain || winbindd_internal_child(child)) {
 			continue;
@@ -708,7 +704,7 @@ void winbind_msg_online(struct messaging_context *msg_ctx,
 
 		messaging_send_buf(msg_ctx, pid_to_procid(child->pid),
 				   MSG_WINBIND_ONLINE,
-				   (uint8 *)child->domain->name,
+				   (const uint8_t *)child->domain->name,
 				   strlen(child->domain->name)+1);
 	}
 }
@@ -770,7 +766,7 @@ void winbind_msg_onlinestatus(struct messaging_context *msg_ctx,
 	}
 
 	messaging_send_buf(msg_ctx, *sender, MSG_WINBIND_ONLINESTATUS, 
-			   (uint8 *)message, strlen(message) + 1);
+			   (const uint8 *)message, strlen(message) + 1);
 
 	talloc_destroy(mem_ctx);
 }
@@ -787,7 +783,7 @@ void winbind_msg_dump_event_list(struct messaging_context *msg_ctx,
 
 	dump_event_list(winbind_event_context());
 
-	for (child = children; child != NULL; child = child->next) {
+	for (child = winbindd_children; child != NULL; child = child->next) {
 
 		DEBUG(10,("winbind_msg_dump_event_list: sending message to pid %u\n",
 			(unsigned int)child->pid));
@@ -847,7 +843,7 @@ void winbind_msg_dump_domain_list(struct messaging_context *msg_ctx,
 
 		messaging_send_buf(msg_ctx, *sender,
 				   MSG_WINBIND_DUMP_DOMAIN_LIST,
-				   (uint8_t *)message, strlen(message) + 1);
+				   (const uint8_t *)message, strlen(message) + 1);
 
 		talloc_destroy(mem_ctx);
 
@@ -881,8 +877,8 @@ void winbind_msg_dump_domain_list(struct messaging_context *msg_ctx,
 	talloc_destroy(mem_ctx);
 }
 
-static void account_lockout_policy_handler(struct event_context *ctx,
-					   struct timed_event *te,
+static void account_lockout_policy_handler(struct tevent_context *ctx,
+					   struct tevent_timer *te,
 					   struct timeval now,
 					   void *private_data)
 {
@@ -920,7 +916,7 @@ static void account_lockout_policy_handler(struct event_context *ctx,
 			 nt_errstr(result)));
 	}
 
-	child->lockout_policy_event = event_add_timed(winbind_event_context(), NULL,
+	child->lockout_policy_event = tevent_add_timer(winbind_event_context(), NULL,
 						      timeval_current_ofs(3600, 0),
 						      account_lockout_policy_handler,
 						      child);
@@ -998,8 +994,8 @@ static bool calculate_next_machine_pwd_change(const char *domain,
 	return true;
 }
 
-static void machine_password_change_handler(struct event_context *ctx,
-					    struct timed_event *te,
+static void machine_password_change_handler(struct tevent_context *ctx,
+					    struct tevent_timer *te,
 					    struct timeval now,
 					    void *private_data)
 {
@@ -1082,7 +1078,7 @@ static void machine_password_change_handler(struct event_context *ctx,
 	}
 
 done:
-	child->machine_password_change_event = event_add_timed(winbind_event_context(), NULL,
+	child->machine_password_change_event = tevent_add_timer(winbind_event_context(), NULL,
 							      next_change,
 							      machine_password_change_handler,
 							      child);
@@ -1195,16 +1191,20 @@ static void child_msg_dump_event_list(struct messaging_context *msg,
 	dump_event_list(winbind_event_context());
 }
 
-bool winbindd_reinit_after_fork(const char *logfilename)
+NTSTATUS winbindd_reinit_after_fork(const struct winbindd_child *myself,
+				    const char *logfilename)
 {
 	struct winbindd_domain *domain;
 	struct winbindd_child *cl;
+	NTSTATUS status;
 
-	if (!NT_STATUS_IS_OK(reinit_after_fork(winbind_messaging_context(),
-					       winbind_event_context(),
-					       true))) {
+	status = reinit_after_fork(
+		winbind_messaging_context(),
+		winbind_event_context(),
+		true);
+	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0,("reinit_after_fork() failed\n"));
-		return false;
+		return status;
 	}
 
 	close_conns_after_fork();
@@ -1215,10 +1215,10 @@ bool winbindd_reinit_after_fork(const char *logfilename)
 	}
 
 	if (!winbindd_setup_sig_term_handler(false))
-		return false;
+		return NT_STATUS_NO_MEMORY;
 	if (!winbindd_setup_sig_hup_handler(override_logfile ? NULL :
 					    logfilename))
-		return false;
+		return NT_STATUS_NO_MEMORY;
 
 	/* Stop zombies in children */
 	CatchChild();
@@ -1257,7 +1257,7 @@ bool winbindd_reinit_after_fork(const char *logfilename)
 	ccache_remove_all_after_fork();
 
 	/* Destroy all possible events in child list. */
-	for (cl = children; cl != NULL; cl = cl->next) {
+	for (cl = winbindd_children; cl != NULL; cl = cl->next) {
 		TALLOC_FREE(cl->lockout_policy_event);
 		TALLOC_FREE(cl->machine_password_change_event);
 
@@ -1266,6 +1266,14 @@ bool winbindd_reinit_after_fork(const char *logfilename)
 		 * go through the parent.
 		 */
 		cl->pid = (pid_t)0;
+
+		/*
+		 * Close service sockets to all other children
+		 */
+		if ((cl != myself) && (cl->sock != -1)) {
+			close(cl->sock);
+			cl->sock = -1;
+		}
         }
 	/*
 	 * This is a little tricky, children must not
@@ -1286,7 +1294,7 @@ bool winbindd_reinit_after_fork(const char *logfilename)
 	cl = idmap_child();
 	cl->pid = (pid_t)0;
 
-	return true;
+	return NT_STATUS_OK;
 }
 
 /*
@@ -1299,13 +1307,73 @@ struct winbindd_domain *wb_child_domain(void)
 	return child_domain;
 }
 
+struct child_handler_state {
+	struct winbindd_child *child;
+	struct winbindd_cli_state cli;
+};
+
+static void child_handler(struct tevent_context *ev, struct tevent_fd *fde,
+			  uint16_t flags, void *private_data)
+{
+	struct child_handler_state *state =
+		(struct child_handler_state *)private_data;
+	NTSTATUS status;
+	struct iovec iov[2];
+	int iov_count;
+
+	/* fetch a request from the main daemon */
+	status = child_read_request(&state->cli);
+
+	if (!NT_STATUS_IS_OK(status)) {
+		/* we lost contact with our parent */
+		_exit(0);
+	}
+
+	DEBUG(4,("child daemon request %d\n",
+		 (int)state->cli.request->cmd));
+
+	ZERO_STRUCTP(state->cli.response);
+	state->cli.request->null_term = '\0';
+	state->cli.mem_ctx = talloc_tos();
+	child_process_request(state->child, &state->cli);
+
+	DEBUG(4, ("Finished processing child request %d\n",
+		  (int)state->cli.request->cmd));
+
+	SAFE_FREE(state->cli.request->extra_data.data);
+
+	iov[0].iov_base = (void *)state->cli.response;
+	iov[0].iov_len = sizeof(struct winbindd_response);
+	iov_count = 1;
+
+	if (state->cli.response->length >
+	    sizeof(struct winbindd_response)) {
+		iov[1].iov_base =
+			(void *)state->cli.response->extra_data.data;
+		iov[1].iov_len = state->cli.response->length-iov[0].iov_len;
+		iov_count = 2;
+	}
+
+	DEBUG(10, ("Writing %d bytes to parent\n",
+		   (int)state->cli.response->length));
+
+	if (write_data_iov(state->cli.sock, iov, iov_count) !=
+	    state->cli.response->length) {
+		DEBUG(0, ("Could not write result\n"));
+		exit(1);
+	}
+}
+
 static bool fork_domain_child(struct winbindd_child *child)
 {
 	int fdpair[2];
-	struct winbindd_cli_state state;
+	struct child_handler_state state;
 	struct winbindd_request request;
 	struct winbindd_response response;
 	struct winbindd_domain *primary_domain = NULL;
+	NTSTATUS status;
+	ssize_t nwritten;
+	struct tevent_fd *fde;
 
 	if (child->domain) {
 		DEBUG(10, ("fork_domain_child called for domain '%s'\n",
@@ -1313,7 +1381,6 @@ static bool fork_domain_child(struct winbindd_child *child)
 	} else {
 		DEBUG(10, ("fork_domain_child called without domain.\n"));
 	}
-	child_domain = child->domain;
 
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fdpair) != 0) {
 		DEBUG(0, ("Could not open child pipe: %s\n",
@@ -1322,34 +1389,67 @@ static bool fork_domain_child(struct winbindd_child *child)
 	}
 
 	ZERO_STRUCT(state);
-	state.pid = sys_getpid();
-	state.request = &request;
-	state.response = &response;
+	state.child = child;
+	state.cli.pid = getpid();
+	state.cli.request = &request;
+	state.cli.response = &response;
 
-	child->pid = sys_fork();
+	child->pid = fork();
 
 	if (child->pid == -1) {
 		DEBUG(0, ("Could not fork: %s\n", strerror(errno)));
+		close(fdpair[0]);
+		close(fdpair[1]);
 		return False;
 	}
 
 	if (child->pid != 0) {
 		/* Parent */
+		ssize_t nread;
+
 		close(fdpair[0]);
+
+		nread = sys_read(fdpair[1], &status, sizeof(status));
+		if (nread != sizeof(status)) {
+			DEBUG(1, ("fork_domain_child: Could not read child status: "
+				  "nread=%d, error=%s\n", (int)nread,
+				  strerror(errno)));
+			close(fdpair[1]);
+			return false;
+		}
+		if (!NT_STATUS_IS_OK(status)) {
+			DEBUG(1, ("fork_domain_child: Child status is %s\n",
+				  nt_errstr(status)));
+			close(fdpair[1]);
+			return false;
+		}
+
 		child->next = child->prev = NULL;
-		DLIST_ADD(children, child);
+		DLIST_ADD(winbindd_children, child);
 		child->sock = fdpair[1];
 		return True;
 	}
 
 	/* Child */
+	child_domain = child->domain;
 
-	DEBUG(10, ("Child process %d\n", (int)sys_getpid()));
+	DEBUG(10, ("Child process %d\n", (int)getpid()));
 
-	state.sock = fdpair[0];
+	state.cli.sock = fdpair[0];
 	close(fdpair[1]);
 
-	if (!winbindd_reinit_after_fork(child->logfilename)) {
+	status = winbindd_reinit_after_fork(child, child->logfilename);
+
+	nwritten = sys_write(state.cli.sock, &status, sizeof(status));
+	if (nwritten != sizeof(status)) {
+		DEBUG(1, ("fork_domain_child: Could not write status: "
+			  "nwritten=%d, error=%s\n", (int)nwritten,
+			  strerror(errno)));
+		_exit(0);
+	}
+	if (!NT_STATUS_IS_OK(status)) {
+		DEBUG(1, ("winbindd_reinit_after_fork failed: %s\n",
+			  nt_errstr(status)));
 		_exit(0);
 	}
 
@@ -1362,6 +1462,10 @@ static bool fork_domain_child(struct winbindd_child *child)
 			   MSG_DUMP_EVENT_LIST, child_msg_dump_event_list);
 	messaging_register(winbind_messaging_context(), NULL,
 			   MSG_DEBUG, debug_message);
+	messaging_register(winbind_messaging_context(), NULL,
+			   MSG_WINBIND_IP_DROPPED,
+			   winbind_msg_ip_dropped);
+
 
 	primary_domain = find_our_domain();
 
@@ -1373,7 +1477,7 @@ static bool fork_domain_child(struct winbindd_child *child)
 	 * try to bring domain online after fork. */
 	if ( child->domain ) {
 		child->domain->startup = True;
-		child->domain->startup_time = time(NULL);
+		child->domain->startup_time = time_mono(NULL);
 		/* we can be in primary domain or in trusted domain
 		 * If we are in trusted domain, set the primary domain
 		 * in start-up mode */
@@ -1381,7 +1485,7 @@ static bool fork_domain_child(struct winbindd_child *child)
 			set_domain_online_request(child->domain);
 			if (!(child->domain->primary)) {
 				primary_domain->startup = True;
-				primary_domain->startup_time = time(NULL);
+				primary_domain->startup_time = time_mono(NULL);
 				set_domain_online_request(primary_domain);
 			}
 		}
@@ -1413,7 +1517,7 @@ static bool fork_domain_child(struct winbindd_child *child)
 			set_domain_online_request(primary_domain);
 		}
 
-		child->lockout_policy_event = event_add_timed(
+		child->lockout_policy_event = tevent_add_timer(
 			winbind_event_context(), NULL, timeval_zero(),
 			account_lockout_policy_handler,
 			child);
@@ -1427,136 +1531,58 @@ static bool fork_domain_child(struct winbindd_child *child)
 
 		if (calculate_next_machine_pwd_change(child->domain->name,
 						       &next_change)) {
-			child->machine_password_change_event = event_add_timed(
+			child->machine_password_change_event = tevent_add_timer(
 				winbind_event_context(), NULL, next_change,
 				machine_password_change_handler,
 				child);
 		}
 	}
 
+	fde = tevent_add_fd(winbind_event_context(), NULL, state.cli.sock,
+			    TEVENT_FD_READ, child_handler, &state);
+	if (fde == NULL) {
+		DEBUG(1, ("tevent_add_fd failed\n"));
+		_exit(1);
+	}
+
 	while (1) {
 
 		int ret;
-		fd_set r_fds;
-		fd_set w_fds;
-		int maxfd;
-		struct timeval t;
-		struct timeval *tp;
-		struct timeval now;
 		TALLOC_CTX *frame = talloc_stackframe();
-		struct iovec iov[2];
-		int iov_count;
-		NTSTATUS status;
 
-		if (run_events(winbind_event_context(), 0, NULL, NULL)) {
-			TALLOC_FREE(frame);
-			continue;
+		ret = tevent_loop_once(winbind_event_context());
+		if (ret != 0) {
+			DEBUG(1, ("tevent_loop_once failed: %s\n",
+				  strerror(errno)));
+			_exit(1);
 		}
 
-		GetTimeOfDay(&now);
-
 		if (child->domain && child->domain->startup &&
-				(now.tv_sec > child->domain->startup_time + 30)) {
+				(time_mono(NULL) > child->domain->startup_time + 30)) {
 			/* No longer in "startup" mode. */
 			DEBUG(10,("fork_domain_child: domain %s no longer in 'startup' mode.\n",
 				child->domain->name ));
 			child->domain->startup = False;
 		}
 
-		FD_ZERO(&r_fds);
-		FD_ZERO(&w_fds);
-
-		if (state.sock < 0 || state.sock >= FD_SETSIZE) {
-			TALLOC_FREE(frame);
-			perror("EBADF");
-			_exit(1);
-		}
-
-		FD_SET(state.sock, &r_fds);
-		maxfd = state.sock;
-
-		/*
-		 * Initialize this high as event_add_to_select_args()
-		 * uses a timeval_min() on this and next_event. Fix
-		 * from Roel van Meer <rolek@alt001.com>.
-		 */
-		t.tv_sec = 999999;
-		t.tv_usec = 0;
-
-		event_add_to_select_args(winbind_event_context(), &now,
-					 &r_fds, &w_fds, &t, &maxfd);
-		tp = get_timed_events_timeout(winbind_event_context(), &t);
-		if (tp) {
-			DEBUG(11,("select will use timeout of %u.%u seconds\n",
-				(unsigned int)tp->tv_sec, (unsigned int)tp->tv_usec ));
-		}
-
-		ret = sys_select(maxfd + 1, &r_fds, &w_fds, NULL, tp);
-
-		if (run_events(winbind_event_context(), ret, &r_fds, &w_fds)) {
-			/* We got a signal - continue. */
-			TALLOC_FREE(frame);
-			continue;
-		}
-
-		if (ret == 0) {
-			DEBUG(11,("nothing is ready yet, continue\n"));
-			TALLOC_FREE(frame);
-			continue;
-		}
-
-		if (ret == -1 && errno == EINTR) {
-			/* We got a signal - continue. */
-			TALLOC_FREE(frame);
-			continue;
-		}
-
-		if (ret == -1 && errno != EINTR) {
-			DEBUG(0,("select error occured\n"));
-			TALLOC_FREE(frame);
-			perror("select");
-			_exit(1);
-		}
-
-		/* fetch a request from the main daemon */
-		status = child_read_request(&state);
-
-		if (!NT_STATUS_IS_OK(status)) {
-			/* we lost contact with our parent */
-			_exit(0);
-		}
-
-		DEBUG(4,("child daemon request %d\n", (int)state.request->cmd));
-
-		ZERO_STRUCTP(state.response);
-		state.request->null_term = '\0';
-		state.mem_ctx = frame;
-		child_process_request(child, &state);
-
-		DEBUG(4, ("Finished processing child request %d\n",
-			  (int)state.request->cmd));
-
-		SAFE_FREE(state.request->extra_data.data);
-
-		iov[0].iov_base = (void *)state.response;
-		iov[0].iov_len = sizeof(struct winbindd_response);
-		iov_count = 1;
-
-		if (state.response->length > sizeof(struct winbindd_response)) {
-			iov[1].iov_base =
-				(void *)state.response->extra_data.data;
-			iov[1].iov_len = state.response->length-iov[0].iov_len;
-			iov_count = 2;
-		}
-
-		DEBUG(10, ("Writing %d bytes to parent\n",
-			   (int)state.response->length));
-
-		if (write_data_iov(state.sock, iov, iov_count) !=
-		    state.response->length) {
-			DEBUG(0, ("Could not write result\n"));
-			exit(1);
-		}
 		TALLOC_FREE(frame);
 	}
 }
+
+void winbind_msg_ip_dropped_parent(struct messaging_context *msg_ctx,
+				   void *private_data,
+				   uint32_t msg_type,
+				   struct server_id server_id,
+				   DATA_BLOB *data)
+{
+	struct winbindd_child *child;
+
+	winbind_msg_ip_dropped(msg_ctx, private_data, msg_type,
+			       server_id, data);
+
+
+	for (child = winbindd_children; child != NULL; child = child->next) {
+		messaging_send_buf(msg_ctx, pid_to_procid(child->pid),
+				   msg_type, data->data, data->length);
+	}
+}
