@@ -27,12 +27,14 @@
 */
 
 #include "includes.h"
-#include "ldb/include/ldb_module.h"
+#include <ldb_module.h>
 #include "ldb/ldb_map/ldb_map.h"
 
 #include "librpc/gen_ndr/ndr_misc.h"
 #include "librpc/ndr/libndr.h"
 #include "dsdb/samdb/samdb.h"
+#include "dsdb/common/util.h"
+#include <ldb_handlers.h>
 
 struct entryuuid_private {
 	struct ldb_context *ldb;
@@ -43,16 +45,14 @@ static struct ldb_val encode_guid(struct ldb_module *module, TALLOC_CTX *ctx, co
 {
 	struct GUID guid;
 	NTSTATUS status = GUID_from_data_blob(val, &guid);
-	enum ndr_err_code ndr_err;
 	struct ldb_val out = data_blob(NULL, 0);
 
 	if (!NT_STATUS_IS_OK(status)) {
 		return out;
 	}
-	ndr_err = ndr_push_struct_blob(&out, ctx, NULL, &guid,
-				       (ndr_push_flags_fn_t)ndr_push_GUID);
-	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-		return out;
+	status = GUID_to_ndr_blob(&guid, ctx, &out);
+	if (!NT_STATUS_IS_OK(status)) {
+		return data_blob(NULL, 0);
 	}
 
 	return out;
@@ -73,16 +73,14 @@ static struct ldb_val encode_ns_guid(struct ldb_module *module, TALLOC_CTX *ctx,
 {
 	struct GUID guid;
 	NTSTATUS status = NS_GUID_from_string((char *)val->data, &guid);
-	enum ndr_err_code ndr_err;
 	struct ldb_val out = data_blob(NULL, 0);
 
 	if (!NT_STATUS_IS_OK(status)) {
 		return out;
 	}
-	ndr_err = ndr_push_struct_blob(&out, ctx, NULL, &guid,
-				       (ndr_push_flags_fn_t)ndr_push_GUID);
-	if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-		return out;
+	status = GUID_to_ndr_blob(&guid, ctx, &out);
+	if (!NT_STATUS_IS_OK(status)) {
+		return data_blob(NULL, 0);
 	}
 
 	return out;
@@ -122,6 +120,25 @@ static struct ldb_val sid_always_binary(struct ldb_module *module, TALLOC_CTX *c
 	return out;
 }
 
+/* Ensure we always convert sids into string, so the backend doesn't have to know about both forms */
+static struct ldb_val sid_always_string(struct ldb_module *module, TALLOC_CTX *ctx, const struct ldb_val *val)
+{
+	struct ldb_context *ldb = ldb_module_get_ctx(module);
+	struct ldb_val out = data_blob(NULL, 0);
+
+	if (ldif_comparision_objectSid_isString(val)) {
+		if (ldb_handler_copy(ldb, ctx, val, &out) != LDB_SUCCESS) {
+			return data_blob(NULL, 0);
+		}
+
+	} else {
+		if (ldif_write_objectSid(ldb, ctx, val, &out) != LDB_SUCCESS) {
+			return data_blob(NULL, 0);
+		}
+	}
+	return out;
+}
+
 /* Ensure we always convert objectCategory into a DN */
 static struct ldb_val objectCategory_always_dn(struct ldb_module *module, TALLOC_CTX *ctx, const struct ldb_val *val)
 {
@@ -131,7 +148,7 @@ static struct ldb_val objectCategory_always_dn(struct ldb_module *module, TALLOC
 	const struct ldb_schema_attribute *a = ldb_schema_attribute_by_name(ldb, "objectCategory");
 
 	dn = ldb_dn_from_ldb_val(ctx, ldb, val);
-	if (dn && ldb_dn_validate(dn)) {
+	if (ldb_dn_validate(dn)) {
 		talloc_free(dn);
 		return val_copy(module, ctx, val);
 	}
@@ -165,7 +182,7 @@ static struct ldb_val usn_to_entryCSN(struct ldb_module *module, TALLOC_CTX *ctx
 
 static unsigned long long entryCSN_to_usn_int(TALLOC_CTX *ctx, const struct ldb_val *val) 
 {
-	char *entryCSN = talloc_strdup(ctx, (const char *)val->data);
+	char *entryCSN = talloc_strndup(ctx, (const char *)val->data, val->length);
 	char *mod_per_sec;
 	time_t t;
 	unsigned long long usn;
@@ -216,10 +233,10 @@ static struct ldb_val usn_to_timestamp(struct ldb_module *module, TALLOC_CTX *ct
 static struct ldb_val timestamp_to_usn(struct ldb_module *module, TALLOC_CTX *ctx, const struct ldb_val *val)
 {
 	struct ldb_val out;
-	time_t t;
+	time_t t=0;
 	unsigned long long usn;
 
-	t = ldb_string_to_time((const char *)val->data);
+	ldb_val_to_time(val, &t);
 	
 	usn = ((unsigned long long)t <<24);
 
@@ -233,7 +250,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	/* objectGUID */
 	{
 		.local_name = "objectGUID",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				.remote_name = "entryUUID", 
@@ -245,7 +262,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	/* invocationId */
 	{
 		.local_name = "invocationId",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				.remote_name = "invocationId", 
@@ -257,7 +274,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	/* objectSid */
 	{
 		.local_name = "objectSid",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				.remote_name = "objectSid", 
@@ -266,18 +283,30 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 			},
 		},
 	},
+	/* securityIdentifier */
+	{
+		.local_name = "securityIdentifier",
+		.type = LDB_MAP_CONVERT,
+		.u = {
+			.convert = {
+				.remote_name = "securityIdentifier",
+				.convert_local = sid_always_binary,
+				.convert_remote = val_copy,
+			},
+		},
+	},
 	{
 		.local_name = "name",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
-				 .remote_name = "samba4RDN"
+				 .remote_name = "rdnValue"
 			 }
 		}
 	},
 	{
 		.local_name = "whenCreated",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "createTimestamp"
@@ -286,7 +315,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "whenChanged",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "modifyTimestamp"
@@ -295,7 +324,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "objectClasses",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "samba4ObjectClasses"
@@ -304,7 +333,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "dITContentRules",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "samba4DITContentRules"
@@ -313,7 +342,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "attributeTypes",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "samba4AttributeTypes"
@@ -322,7 +351,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "objectCategory",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				.remote_name = "objectCategory", 
@@ -333,7 +362,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "distinguishedName",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "entryDN"
@@ -342,7 +371,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "primaryGroupID",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "primaryGroupID",
@@ -353,7 +382,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "groupType",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "groupType",
@@ -364,7 +393,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "userAccountControl",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "userAccountControl",
@@ -375,7 +404,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "sAMAccountType",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "sAMAccountType",
@@ -386,7 +415,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "systemFlags",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "systemFlags",
@@ -397,7 +426,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "usnChanged",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "entryCSN",
@@ -408,7 +437,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "usnCreated",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "createTimestamp",
@@ -419,7 +448,7 @@ static const struct ldb_map_attribute entryuuid_attributes[] =
 	},
 	{
 		.local_name = "*",
-		.type = MAP_KEEP,
+		.type = LDB_MAP_KEEP,
 	},
 	{
 		.local_name = NULL,
@@ -455,7 +484,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	/* objectGUID */
 	{
 		.local_name = "objectGUID",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				.remote_name = "nsuniqueid", 
@@ -467,18 +496,30 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	/* objectSid */	
 	{
 		.local_name = "objectSid",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
-				.remote_name = "objectSid", 
-				.convert_local = sid_always_binary,
-				.convert_remote = val_copy,
+				.remote_name = "sambaSID", 
+				.convert_local = sid_always_string,
+				.convert_remote = sid_always_binary,
 			}
 		}
 	},
+	/* securityIdentifier */
+	{
+		.local_name = "securityIdentifier",
+		.type = LDB_MAP_CONVERT,
+		.u = {
+			.convert = {
+				.remote_name = "securityIdentifier",
+				.convert_local = sid_always_binary,
+				.convert_remote = val_copy,
+			},
+		},
+	},
 	{
 		.local_name = "whenCreated",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "createTimestamp"
@@ -487,7 +528,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "whenChanged",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "modifyTimestamp"
@@ -496,7 +537,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "objectCategory",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				.remote_name = "objectCategory", 
@@ -507,7 +548,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "distinguishedName",
-		.type = MAP_RENAME,
+		.type = LDB_MAP_RENAME,
 		.u = {
 			.rename = {
 				 .remote_name = "entryDN"
@@ -516,7 +557,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "primaryGroupID",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "primaryGroupID",
@@ -527,10 +568,10 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "groupType",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
-				 .remote_name = "groupType",
+				 .remote_name = "sambaGroupType",
 				 .convert_local = normalise_to_signed32,
 				 .convert_remote = val_copy,
 			 }
@@ -538,7 +579,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "userAccountControl",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "userAccountControl",
@@ -549,7 +590,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "sAMAccountType",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "sAMAccountType",
@@ -560,7 +601,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "systemFlags",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "systemFlags",
@@ -571,7 +612,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "usnChanged",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "modifyTimestamp",
@@ -582,7 +623,7 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 	},
 	{
 		.local_name = "usnCreated",
-		.type = MAP_CONVERT,
+		.type = LDB_MAP_CONVERT,
 		.u = {
 			.convert = {
 				 .remote_name = "createTimestamp",
@@ -591,15 +632,131 @@ static const struct ldb_map_attribute nsuniqueid_attributes[] =
 			 }
 		}
 	},
+	{
+		.local_name = "pwdLastSet",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaPwdLastSet"
+			 }
+		}
+	},
+	{
+		.local_name = "lastLogon",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaLogonTime"
+			 }
+		}
+	},
+	{
+		.local_name = "lastLogoff",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaLogoffTime"
+			 }
+		}
+	},
+	{
+		.local_name = "badPwdCount",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaBadPasswordCount"
+			 }
+		}
+	},
+	{
+		.local_name = "logonHours",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaLogonHours"
+			 }
+		}
+	},
+	{
+		.local_name = "homeDrive",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaHomeDrive"
+			 }
+		}
+	},
+	{
+		.local_name = "scriptPath",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaLogonScript"
+			 }
+		}
+	},
+	{
+		.local_name = "profilePath",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaProfilePath"
+			 }
+		}
+	},
+	{
+		.local_name = "userWorkstations",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaUserWorkstations"
+			 }
+		}
+	},
+	{
+		.local_name = "homeDirectory",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaHomePath"
+			 }
+		}
+	},
+	{
+		.local_name = "nextRid",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaNextRid"
+			 }
+		}
+	},
+	{
+		.local_name = "privilegeDisplayName",
+		.type = LDB_MAP_RENAME,
+		.u = {
+			.rename = {
+				 .remote_name = "sambaPrivName"
+			 }
+		}
+	},
 	{
 		.local_name = "*",
-		.type = MAP_KEEP,
+		.type = LDB_MAP_KEEP,
 	},
 	{
 		.local_name = NULL,
 	}
 };
 
+/* This objectClass conflicts with builtin classes on FDS */
+const struct ldb_map_objectclass nsuniqueid_objectclasses[] =
+{
+	{
+		.local_name = NULL
+	}
+};
+
 /* These things do not show up in wildcard searches in OpenLDAP, but
  * we need them to show up in the AD-like view */
 static const char * const nsuniqueid_wildcard_attributes[] = {
@@ -626,7 +783,7 @@ static int entryuuid_init(struct ldb_module *module)
 static int nsuniqueid_init(struct ldb_module *module)
 {
         int ret;
-	ret = ldb_map_init(module, nsuniqueid_attributes, NULL, nsuniqueid_wildcard_attributes, "extensibleObject", NULL);
+	ret = ldb_map_init(module, nsuniqueid_attributes, nsuniqueid_objectclasses, nsuniqueid_wildcard_attributes, "extensibleObject", NULL);
         if (ret != LDB_SUCCESS)
                 return ret;
 
@@ -665,7 +822,6 @@ static int entryuuid_sequence_number(struct ldb_module *module, struct ldb_reque
 	struct ldb_context *ldb;
 	int ret;
 	struct map_private *map_private;
-	struct entryuuid_private *entryuuid_private;
 	unsigned long long seq_num = 0;
 	struct ldb_request *search_req;
 
@@ -685,26 +841,34 @@ static int entryuuid_sequence_number(struct ldb_module *module, struct ldb_reque
 	seq = talloc_get_type(req->op.extended.data, struct ldb_seqnum_request);
 
 	map_private = talloc_get_type(ldb_module_get_private(module), struct map_private);
-
-	entryuuid_private = talloc_get_type(map_private->caller_private, struct entryuuid_private);
+	if (!map_private) {
+		ldb_debug_set(ldb, LDB_DEBUG_FATAL,
+			      "private data is not of type struct map_private");
+		return LDB_ERR_PROTOCOL_ERROR;
+	}
 
 	/* All this to get the DN of the parition, so we can search the right thing */
 	partition_ctrl = ldb_request_get_control(req, DSDB_CONTROL_CURRENT_PARTITION_OID);
 	if (!partition_ctrl) {
 		ldb_debug_set(ldb, LDB_DEBUG_FATAL,
-			      "entryuuid_sequence_number: no current partition control found");
-		return LDB_ERR_CONSTRAINT_VIOLATION;
+			      "entryuuid_sequence_number: no current partition control found!");
+		return LDB_ERR_PROTOCOL_ERROR;
 	}
 
 	partition = talloc_get_type(partition_ctrl->data,
 				    struct dsdb_control_current_partition);
-	SMB_ASSERT(partition && partition->version == DSDB_CONTROL_CURRENT_PARTITION_VERSION);
+	if ((partition == NULL) || (partition->version != DSDB_CONTROL_CURRENT_PARTITION_VERSION)) {
+		ldb_debug_set(ldb, LDB_DEBUG_FATAL,
+			      "entryuuid_sequence_number: current partition control with wrong data!");
+		return LDB_ERR_PROTOCOL_ERROR;
+	}
 
 	ret = ldb_build_search_req(&search_req, ldb, req,
 				   partition->dn, LDB_SCOPE_BASE,
 				   NULL, contextCSN_attr, NULL,
 				   &seq_num, get_seq_callback,
 				   NULL);
+	LDB_REQ_SET_LOCATION(search_req);
 	if (ret != LDB_SUCCESS) {
 		return ret;
 	}
@@ -722,12 +886,12 @@ static int entryuuid_sequence_number(struct ldb_module *module, struct ldb_reque
 
 	ext = talloc_zero(req, struct ldb_extended);
 	if (!ext) {
-		return LDB_ERR_OPERATIONS_ERROR;
+		return ldb_oom(ldb);
 	}
 	seqr = talloc_zero(req, struct ldb_seqnum_result);
 	if (seqr == NULL) {
 		talloc_free(ext);
-		return LDB_ERR_OPERATIONS_ERROR;
+		return ldb_oom(ldb);
 	}
 	ext->oid = LDB_EXTENDED_SEQUENCE_NUMBER;
 	ext->data = seqr;
@@ -741,13 +905,10 @@ static int entryuuid_sequence_number(struct ldb_module *module, struct ldb_reque
 		seqr->seq_num++;
 		break;
 	case LDB_SEQ_HIGHEST_TIMESTAMP:
-	{
-		seqr->seq_num = (seq_num >> 24);
-		break;
-	}
+		return ldb_module_error(module, LDB_ERR_OPERATIONS_ERROR, "LDB_SEQ_HIGHEST_TIMESTAMP not supported");
 	}
+
 	seqr->flags = 0;
-	seqr->flags |= LDB_SEQ_TIMESTAMP_SEQUENCE;
 	seqr->flags |= LDB_SEQ_GLOBAL_SEQUENCE;
 
 	/* send request done */
@@ -763,16 +924,34 @@ static int entryuuid_extended(struct ldb_module *module, struct ldb_request *req
 	return ldb_next_request(module, req);
 }
 
-_PUBLIC_ const struct ldb_module_ops ldb_entryuuid_module_ops = {
+static const struct ldb_module_ops ldb_entryuuid_module_ops = {
 	.name		   = "entryuuid",
 	.init_context	   = entryuuid_init,
 	.extended          = entryuuid_extended,
 	LDB_MAP_OPS
 };
 
-_PUBLIC_ const struct ldb_module_ops ldb_nsuniqueid_module_ops = {
+static const struct ldb_module_ops ldb_nsuniqueid_module_ops = {
 	.name		   = "nsuniqueid",
 	.init_context	   = nsuniqueid_init,
 	.extended          = entryuuid_extended,
 	LDB_MAP_OPS
 };
+
+/*
+  initialise the module
+ */
+_PUBLIC_ int ldb_simple_ldap_map_module_init(const char *version)
+{
+	int ret;
+	LDB_MODULE_CHECK_VERSION(version);
+	ret = ldb_register_module(&ldb_entryuuid_module_ops);
+	if (ret != LDB_SUCCESS) {
+		return ret;
+	}
+	ret = ldb_register_module(&ldb_nsuniqueid_module_ops);
+	if (ret != LDB_SUCCESS) {
+		return ret;
+	}
+	return LDB_SUCCESS;
+}
