@@ -24,12 +24,14 @@
 #include "system/kerberos.h"
 #include "auth/auth.h"
 #include "auth/kerberos/kerberos.h"
-#include "librpc/gen_ndr/ndr_krb5pac.h"
 #include "samba3/samba3.h"
 #include "libcli/security/security.h"
 #include "torture/torture.h"
 #include "auth/auth_sam_reply.h"
 #include "param/param.h"
+#include "librpc/gen_ndr/ndr_krb5pac.h"
+#include "torture/auth/proto.h"
+#include "auth/kerberos/pac_utils.h"
 
 static bool torture_pac_self_check(struct torture_context *tctx)
 {
@@ -49,8 +51,8 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 
 	struct smb_krb5_context *smb_krb5_context;
 
-	struct auth_serversupplied_info *server_info;
-	struct auth_serversupplied_info *server_info_out;
+	struct auth_user_info_dc *user_info_dc;
+	struct auth_user_info_dc *user_info_dc_out;
 
 	krb5_principal client_principal;
 	time_t logon_time = time(NULL);
@@ -66,7 +68,7 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 	generate_random_buffer(server_bytes, 16);
 	generate_random_buffer(krbtgt_bytes, 16);
 
-	ret = krb5_keyblock_init(smb_krb5_context->krb5_context,
+	ret = smb_krb5_keyblock_init_contents(smb_krb5_context->krb5_context,
 				 ENCTYPE_ARCFOUR_HMAC,
 				 server_bytes, sizeof(server_bytes),
 				 &server_keyblock);
@@ -75,7 +77,7 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 						   smb_get_krb5_error_message(smb_krb5_context->krb5_context, 
 									      ret, mem_ctx)));
 
-	ret = krb5_keyblock_init(smb_krb5_context->krb5_context,
+	ret = smb_krb5_keyblock_init_contents(smb_krb5_context->krb5_context,
 				 ENCTYPE_ARCFOUR_HMAC,
 				 krbtgt_bytes, sizeof(krbtgt_bytes),
 				 &krbtgt_keyblock);
@@ -91,18 +93,18 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 	}
 
 	/* We need an input, and this one requires no underlying database */
-	nt_status = auth_anonymous_server_info(mem_ctx, lp_netbios_name(tctx->lp_ctx), &server_info);
+	nt_status = auth_anonymous_user_info_dc(mem_ctx, lpcfg_netbios_name(tctx->lp_ctx), &user_info_dc);
 
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    &server_keyblock);
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    &krbtgt_keyblock);
-		torture_fail(tctx, "auth_anonymous_server_info");
+		torture_fail(tctx, "auth_anonymous_user_info_dc");
 	}
 
 	ret = krb5_parse_name_flags(smb_krb5_context->krb5_context, 
-				    server_info->account_name, 
+				    user_info_dc->info->account_name,
 				    KRB5_PRINCIPAL_PARSE_NO_REALM, 
 				    &client_principal);
 	if (ret) {
@@ -115,8 +117,7 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 
 	/* OK, go ahead and make a PAC */
 	ret = kerberos_create_pac(mem_ctx, 
-				  lp_iconv_convenience(tctx->lp_ctx),
-				  server_info, 
+				  user_info_dc,
 				  smb_krb5_context->krb5_context,  
 				  &krbtgt_keyblock,
 				  &server_keyblock,
@@ -142,14 +143,13 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 
 	/* Now check that we can read it back (using full decode and validate) */
 	nt_status = kerberos_decode_pac(mem_ctx, 
-					lp_iconv_convenience(tctx->lp_ctx),
-					&pac_data,
 					tmp_blob,
 					smb_krb5_context->krb5_context,
 					&krbtgt_keyblock,
 					&server_keyblock,
 					client_principal, 
-					logon_time, NULL);
+					logon_time,
+ 					&pac_data);
 
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -165,14 +165,14 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 	}
 
 	/* Now check we can read it back (using Heimdal's pac parsing) */
-	nt_status = kerberos_pac_blob_to_server_info(mem_ctx, 
-						     lp_iconv_convenience(tctx->lp_ctx),
+	nt_status = kerberos_pac_blob_to_user_info_dc(mem_ctx,
 						     tmp_blob, 
 						     smb_krb5_context->krb5_context,
-						     &server_info_out);
+						      &user_info_dc_out, NULL, NULL);
 
-	if (!dom_sid_equal(server_info->account_sid, 
-			   server_info_out->account_sid)) {
+	/* The user's SID is the first element in the list */
+	if (!dom_sid_equal(user_info_dc->sids,
+			   user_info_dc_out->sids)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    &krbtgt_keyblock);
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -183,22 +183,20 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 		torture_fail(tctx,  
 			     talloc_asprintf(tctx, 
 					     "(self test) PAC Decode resulted in *different* domain SID: %s != %s",
-					     dom_sid_string(mem_ctx, server_info->account_sid), 
-					     dom_sid_string(mem_ctx, server_info_out->account_sid)));
+					     dom_sid_string(mem_ctx, user_info_dc->sids),
+					     dom_sid_string(mem_ctx, user_info_dc_out->sids)));
 	}
-	talloc_free(server_info_out);
+	talloc_free(user_info_dc_out);
 
 	/* Now check that we can read it back (yet again) */
 	nt_status = kerberos_pac_logon_info(mem_ctx, 
-					    lp_iconv_convenience(tctx->lp_ctx),
-					    &logon_info,
 					    tmp_blob,
 					    smb_krb5_context->krb5_context,
 					    &krbtgt_keyblock,
 					    &server_keyblock,
 					    client_principal, 
 					    logon_time, 
-					    NULL);
+					    &logon_info);
 	
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -223,10 +221,11 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 
 	/* And make a server info from the samba-parsed PAC */
 	validation.sam3 = &logon_info->info3;
-	nt_status = make_server_info_netlogon_validation(mem_ctx,
+	nt_status = make_user_info_dc_netlogon_validation(mem_ctx,
 							 "",
 							 3, &validation,
-							 &server_info_out); 
+							  true, /* This user was authenticated */
+						 &user_info_dc_out);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		torture_fail(tctx, 
 			     talloc_asprintf(tctx, 
@@ -234,13 +233,13 @@ static bool torture_pac_self_check(struct torture_context *tctx)
 					     nt_errstr(nt_status)));
 	}
 	
-	if (!dom_sid_equal(server_info->account_sid, 
-			   server_info_out->account_sid)) {
+	if (!dom_sid_equal(user_info_dc->sids,
+			   user_info_dc_out->sids)) {
 		torture_fail(tctx,  
 			     talloc_asprintf(tctx, 
 					     "(self test) PAC Decode resulted in *different* domain SID: %s != %s",
-					     dom_sid_string(mem_ctx, server_info->account_sid), 
-					     dom_sid_string(mem_ctx, server_info_out->account_sid)));
+					     dom_sid_string(mem_ctx, user_info_dc->sids),
+					     dom_sid_string(mem_ctx, user_info_dc_out->sids)));
 	}
 	return true;
 }
@@ -302,7 +301,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	struct PAC_LOGON_INFO *logon_info;
 	union netr_Validation validation;
 	const char *pac_file, *pac_kdc_key, *pac_member_key;
-	struct auth_serversupplied_info *server_info_out;
+	struct auth_user_info_dc *user_info_dc_out;
 
 	krb5_keyblock server_keyblock;
 	krb5_keyblock krbtgt_keyblock, *krbtgt_keyblock_p;
@@ -349,7 +348,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 		torture_fail(tctx, "(saved test) Could not interpret krbsrv key");
 	}
 
-	ret = krb5_keyblock_init(smb_krb5_context->krb5_context,
+	ret = smb_krb5_keyblock_init_contents(smb_krb5_context->krb5_context,
 				 ENCTYPE_ARCFOUR_HMAC,
 				 krbsrv_bytes->hash, sizeof(krbsrv_bytes->hash),
 				 &server_keyblock);
@@ -360,7 +359,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 								  ret, mem_ctx)));
 
 	if (krbtgt_bytes) {
-		ret = krb5_keyblock_init(smb_krb5_context->krb5_context,
+		ret = smb_krb5_keyblock_init_contents(smb_krb5_context->krb5_context,
 					 ENCTYPE_ARCFOUR_HMAC,
 					 krbtgt_bytes->hash, sizeof(krbtgt_bytes->hash),
 					 &krbtgt_keyblock);
@@ -410,13 +409,11 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 
 	/* Decode and verify the signaure on the PAC */
 	nt_status = kerberos_decode_pac(mem_ctx, 
-					lp_iconv_convenience(tctx->lp_ctx),
-					&pac_data,
 					tmp_blob,
 					smb_krb5_context->krb5_context,
 					krbtgt_keyblock_p,
 					&server_keyblock, 
-					client_principal, authtime, NULL);
+					client_principal, authtime, &pac_data);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    krbtgt_keyblock_p);
@@ -430,11 +427,11 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	}
 
 	/* Now check we can read it back (using Heimdal's pac parsing) */
-	nt_status = kerberos_pac_blob_to_server_info(mem_ctx, 
-						     lp_iconv_convenience(tctx->lp_ctx),
+	nt_status = kerberos_pac_blob_to_user_info_dc(mem_ctx,
 						     tmp_blob, 
 						     smb_krb5_context->krb5_context,
-						     &server_info_out);
+						      &user_info_dc_out,
+						      NULL, NULL);
 
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -451,7 +448,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	if (!pac_file &&
 	    !dom_sid_equal(dom_sid_parse_talloc(mem_ctx, 
 						"S-1-5-21-3048156945-3961193616-3706469200-1005"), 
-			   server_info_out->account_sid)) {
+			   user_info_dc_out->sids)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    krbtgt_keyblock_p);
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -462,20 +459,18 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 			     talloc_asprintf(tctx, 
 					     "(saved test) Heimdal PAC Decode resulted in *different* domain SID: %s != %s",
 					     "S-1-5-21-3048156945-3961193616-3706469200-1005", 
-					     dom_sid_string(mem_ctx, server_info_out->account_sid)));
+					     dom_sid_string(mem_ctx, user_info_dc_out->sids)));
 	}
 
-	talloc_free(server_info_out);
+	talloc_free(user_info_dc_out);
 
 	/* Parse the PAC again, for the logon info this time (using Samba4's parsing) */
 	nt_status = kerberos_pac_logon_info(mem_ctx, 
-					    lp_iconv_convenience(tctx->lp_ctx),
-					    &logon_info,
 					    tmp_blob,
 					    smb_krb5_context->krb5_context,
 					    krbtgt_keyblock_p,
 					    &server_keyblock,
-					    client_principal, authtime, NULL);
+					    client_principal, authtime, &logon_info);
 
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -491,10 +486,11 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	}
 
 	validation.sam3 = &logon_info->info3;
-	nt_status = make_server_info_netlogon_validation(mem_ctx,
+	nt_status = make_user_info_dc_netlogon_validation(mem_ctx,
 							 "",
 							 3, &validation,
-							 &server_info_out); 
+							  true, /* This user was authenticated */
+							 &user_info_dc_out);
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    krbtgt_keyblock_p);
@@ -511,7 +507,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	if (!pac_file &&
 	    !dom_sid_equal(dom_sid_parse_talloc(mem_ctx, 
 						"S-1-5-21-3048156945-3961193616-3706469200-1005"), 
-			   server_info_out->account_sid)) {
+			   user_info_dc_out->sids)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    krbtgt_keyblock_p);
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -522,7 +518,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 			     talloc_asprintf(tctx, 
 					     "(saved test) PAC Decode resulted in *different* domain SID: %s != %s",
 					     "S-1-5-21-3048156945-3961193616-3706469200-1005", 
-					     dom_sid_string(mem_ctx, server_info_out->account_sid)));
+					     dom_sid_string(mem_ctx, user_info_dc_out->sids)));
 	}
 
 	if (krbtgt_bytes == NULL) {
@@ -534,7 +530,6 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	}
 
 	ret = kerberos_encode_pac(mem_ctx, 
-				  lp_iconv_convenience(tctx->lp_ctx),
 				  pac_data,
 				  smb_krb5_context->krb5_context,
 				  krbtgt_keyblock_p,
@@ -586,8 +581,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	}
 
 	ret = kerberos_create_pac(mem_ctx, 
-				  lp_iconv_convenience(tctx->lp_ctx),
-				  server_info_out,
+				  user_info_dc_out,
 				  smb_krb5_context->krb5_context,
 				  krbtgt_keyblock_p,
 				  &server_keyblock,
@@ -612,7 +606,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	 */
 	if (tmp_blob.length != validate_blob.length) {
 		ndr_err = ndr_pull_struct_blob(&validate_blob, mem_ctx, 
-					       lp_iconv_convenience(tctx->lp_ctx), &pac_data2,
+					       &pac_data2,
 					       (ndr_pull_flags_fn_t)ndr_pull_PAC_DATA);
 		nt_status = ndr_map_error2ntstatus(ndr_err);
 		torture_assert_ntstatus_ok(tctx, nt_status, "can't parse the PAC");
@@ -634,7 +628,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 
 	if (memcmp(tmp_blob.data, validate_blob.data, tmp_blob.length) != 0) {
 		ndr_err = ndr_pull_struct_blob(&validate_blob, mem_ctx, 
-					       lp_iconv_convenience(tctx->lp_ctx), &pac_data2,
+					       &pac_data2,
 					       (ndr_pull_flags_fn_t)ndr_pull_PAC_DATA);
 		nt_status = ndr_map_error2ntstatus(ndr_err);
 		torture_assert_ntstatus_ok(tctx, nt_status, "can't parse the PAC");
@@ -660,14 +654,12 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 
 	/* Break the auth time, to ensure we check this vital detail (not setting this caused all the pain in the first place... */
 	nt_status = kerberos_decode_pac(mem_ctx, 
-					lp_iconv_convenience(tctx->lp_ctx),
-					&pac_data,
 					tmp_blob,
 					smb_krb5_context->krb5_context,
 					krbtgt_keyblock_p,
 					&server_keyblock,
 					client_principal, 
-					authtime + 1, NULL);
+					authtime + 1, &pac_data);
 	if (NT_STATUS_IS_OK(nt_status)) {
 
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
@@ -698,14 +690,12 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	}
 
 	nt_status = kerberos_decode_pac(mem_ctx, 
-					lp_iconv_convenience(tctx->lp_ctx),
-					&pac_data,
 					tmp_blob,
 					smb_krb5_context->krb5_context,
 					krbtgt_keyblock_p,
 					&server_keyblock,
 					client_principal, 
-					authtime, NULL);
+					authtime, &pac_data);
 	if (NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    krbtgt_keyblock_p);
@@ -718,14 +708,13 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 	tmp_blob.data[tmp_blob.length - 2]++;
 
 	nt_status = kerberos_decode_pac(mem_ctx, 
-					lp_iconv_convenience(tctx->lp_ctx),
-					&pac_data,
 					tmp_blob,
 					smb_krb5_context->krb5_context,
 					krbtgt_keyblock_p,
 					&server_keyblock,
 					client_principal, 
-					authtime, NULL);
+					authtime,
+					&pac_data);
 	if (NT_STATUS_IS_OK(nt_status)) {
 		krb5_free_keyblock_contents(smb_krb5_context->krb5_context, 
 					    krbtgt_keyblock_p);
@@ -743,7 +732,7 @@ static bool torture_pac_saved_check(struct torture_context *tctx)
 
 struct torture_suite *torture_pac(TALLOC_CTX *mem_ctx)
 {
-	struct torture_suite *suite = torture_suite_create(mem_ctx, "PAC");
+	struct torture_suite *suite = torture_suite_create(mem_ctx, "pac");
 
 	torture_suite_add_simple_test(suite, "self check", 
 				      torture_pac_self_check);
